# حقوق الطبع والنشر محفوظة © 2025 mero - من مدينة رام الله فلسطين

import os
import sys
import django
from django.conf import settings
from django.core.wsgi import get_wsgi_application
from django.http import JsonResponse, HttpResponse
from django.urls import path, include
from django.views.decorators.csrf import csrf_exempt
from django.core.management import execute_from_command_line
from django.core.servers.basehttp import get_internal_wsgi_application
import threading
import time
import json
import psutil
from typing import Dict, Any, List, Optional

class DjangoServer:
    def __init__(self, 
                 app_name: str = "RunMero Django Server",
                 host: str = "0.0.0.0",
                 port: int = 8000,
                 debug: bool = False,
                 auto_reload: bool = False):
        
        self.app_name = app_name
        self.host = host
        self.port = port
        self.debug = debug
        self.auto_reload = auto_reload
        
        self._running = False
        self._server_thread = None
        self._start_time = None
        self._request_count = 0
        
        self._setup_django()
        self._setup_urls()

    def _setup_django(self):
        if not settings.configured:
            settings.configure(
                DEBUG=self.debug,
                SECRET_KEY='runmero-django-server-palestine-mero-2025',
                ALLOWED_HOSTS=['*'],
                ROOT_URLCONF=__name__,
                INSTALLED_APPS=[
                    'django.contrib.contenttypes',
                    'django.contrib.sessions',
                    'django.contrib.messages',
                    'django.contrib.staticfiles',
                ],
                MIDDLEWARE=[
                    'django.middleware.security.SecurityMiddleware',
                    'django.contrib.sessions.middleware.SessionMiddleware',
                    'django.middleware.common.CommonMiddleware',
                    'django.contrib.messages.middleware.MessageMiddleware',
                    'django.middleware.clickjacking.XFrameOptionsMiddleware',
                ],
                DATABASES={
                    'default': {
                        'ENGINE': 'django.db.backends.sqlite3',
                        'NAME': ':memory:',
                    }
                },
                USE_TZ=True,
                USE_I18N=True,
                LANGUAGE_CODE='ar',
                TIME_ZONE='Asia/Jerusalem',
                STATIC_URL='/static/',
                TEMPLATES=[
                    {
                        'BACKEND': 'django.template.backends.django.DjangoTemplates',
                        'DIRS': [],
                        'APP_DIRS': True,
                        'OPTIONS': {
                            'context_processors': [
                                'django.template.context_processors.debug',
                                'django.template.context_processors.request',
                                'django.contrib.messages.context_processors.messages',
                            ],
                        },
                    },
                ],
            )
            
            django.setup()

    def _setup_urls(self):
        def home_view(request):
            self._request_count += 1
            return JsonResponse({
                "message": "مرحباً بك في خادم Django المدعوم بـ RunMero",
                "version": "2.5.0", 
                "author": "mero",
                "country": "فلسطين الحبيبة",
                "status": "running",
                "uptime": time.time() - (self._start_time or time.time()),
                "requests_served": self._request_count,
                "framework": "Django",
                "server": self.app_name
            }, json_dumps_params={'ensure_ascii': False})

        def health_view(request):
            return JsonResponse({
                "status": "healthy",
                "timestamp": time.time(),
                "server": self.app_name,
                "process_id": os.getpid(),
                "django_version": django.get_version()
            }, json_dumps_params={'ensure_ascii': False})

        def status_view(request):
            process = psutil.Process()
            
            return JsonResponse({
                "server_info": {
                    "name": self.app_name,
                    "host": self.host,
                    "port": self.port,
                    "running": self._running,
                    "uptime": time.time() - (self._start_time or time.time()) if self._start_time else 0,
                    "requests": self._request_count,
                    "django_version": django.get_version()
                },
                "system_info": {
                    "cpu_percent": process.cpu_percent(),
                    "memory_info": process.memory_info()._asdict(),
                    "threads": process.num_threads(),
                    "connections": len(process.connections()) if hasattr(process, 'connections') else 0
                },
                "environment": {
                    "python_version": f"{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}",
                    "platform": os.name,
                    "pid": os.getpid()
                }
            }, json_dumps_params={'ensure_ascii': False})

        @csrf_exempt
        def processes_view(request):
            if request.method == 'GET':
                try:
                    processes = []
                    for proc in psutil.process_iter(['pid', 'name', 'cpu_percent', 'memory_percent', 'status']):
                        try:
                            processes.append(proc.info)
                        except (psutil.NoSuchProcess, psutil.AccessDenied):
                            pass
                    
                    return JsonResponse({
                        "processes": processes[:30],
                        "total": len(processes),
                        "timestamp": time.time()
                    }, json_dumps_params={'ensure_ascii': False})
                    
                except Exception as e:
                    return JsonResponse({"error": str(e)}, status=500, json_dumps_params={'ensure_ascii': False})
            
            elif request.method == 'POST':
                try:
                    data = json.loads(request.body)
                    return JsonResponse({
                        "message": f"تم إنشاء العملية: {data.get('name', 'unknown')}",
                        "process": data,
                        "timestamp": time.time()
                    }, json_dumps_params={'ensure_ascii': False})
                except json.JSONDecodeError:
                    return JsonResponse({"error": "بيانات JSON غير صالحة"}, status=400, json_dumps_params={'ensure_ascii': False})

        def system_view(request):
            try:
                return JsonResponse({
                    "cpu": {
                        "percent": psutil.cpu_percent(interval=1),
                        "count": psutil.cpu_count(),
                        "freq": psutil.cpu_freq()._asdict() if psutil.cpu_freq() else None
                    },
                    "memory": psutil.virtual_memory()._asdict(),
                    "disk": psutil.disk_usage('/')._asdict(),
                    "network": {name: stats._asdict() for name, stats in psutil.net_io_counters(pernic=True).items()},
                    "boot_time": psutil.boot_time(),
                    "uptime": time.time() - psutil.boot_time()
                }, json_dumps_params={'ensure_ascii': False})
            except Exception as e:
                return JsonResponse({"error": str(e)}, status=500, json_dumps_params={'ensure_ascii': False})

        def dashboard_view(request):
            dashboard_html = """
            <!DOCTYPE html>
            <html dir="rtl" lang="ar">
            <head>
                <meta charset="UTF-8">
                <meta name="viewport" content="width=device-width, initial-scale=1.0">
                <title>لوحة تحكم RunMero Django</title>
                <style>
                    body { 
                        font-family: 'Segoe UI', Arial, sans-serif; 
                        margin: 0; padding: 20px; 
                        background: linear-gradient(135deg, #667eea 0%, #764ba2 100%);
                        min-height: 100vh;
                    }
                    .container { max-width: 1200px; margin: 0 auto; }
                    .header { 
                        background: rgba(255,255,255,0.95); 
                        color: #333; padding: 30px; 
                        border-radius: 15px; 
                        margin-bottom: 30px; 
                        box-shadow: 0 8px 32px rgba(0,0,0,0.1);
                        backdrop-filter: blur(10px);
                    }
                    .stats { 
                        display: grid; 
                        grid-template-columns: repeat(auto-fit, minmax(280px, 1fr)); 
                        gap: 20px; margin-bottom: 30px; 
                    }
                    .stat-card { 
                        background: rgba(255,255,255,0.95); 
                        padding: 25px; border-radius: 15px; 
                        box-shadow: 0 8px 32px rgba(0,0,0,0.1);
                        backdrop-filter: blur(10px);
                        transition: transform 0.3s ease;
                    }
                    .stat-card:hover { transform: translateY(-5px); }
                    .stat-value { 
                        font-size: 2.5em; font-weight: bold; 
                        color: #667eea; margin: 10px 0;
                        text-shadow: 2px 2px 4px rgba(0,0,0,0.1);
                    }
                    .stat-label { color: #666; font-size: 0.9em; }
                    .info-section { 
                        background: rgba(255,255,255,0.95); 
                        padding: 25px; border-radius: 15px; 
                        box-shadow: 0 8px 32px rgba(0,0,0,0.1);
                        backdrop-filter: blur(10px);
                    }
                    .refresh-btn { 
                        background: linear-gradient(45deg, #667eea, #764ba2); 
                        color: white; border: none; 
                        padding: 12px 24px; border-radius: 25px; 
                        cursor: pointer; font-weight: bold;
                        transition: all 0.3s ease;
                    }
                    .refresh-btn:hover { 
                        transform: scale(1.05); 
                        box-shadow: 0 4px 15px rgba(102, 126, 234, 0.4);
                    }
                    h1 { margin: 0; font-size: 2.5em; }
                    h2 { color: #667eea; }
                    pre { 
                        background: #f8f9fa; padding: 15px; 
                        border-radius: 10px; overflow-x: auto;
                        border-left: 4px solid #667eea;
                    }
                </style>
            </head>
            <body>
                <div class="container">
                    <div class="header">
                        <h1>🚀 لوحة تحكم RunMero Django Server</h1>
                        <p>خادم Django قوي مدعوم بمكتبة RunMero - صنع بحب في فلسطين الحبيبة 🇵🇸</p>
                        <small>Django Version: """ + django.get_version() + """</small>
                    </div>
                    
                    <div class="stats">
                        <div class="stat-card">
                            <div class="stat-label">الطلبات المخدومة</div>
                            <div class="stat-value" id="requests">""" + str(self._request_count) + """</div>
                        </div>
                        <div class="stat-card">
                            <div class="stat-label">وقت التشغيل (ثانية)</div>
                            <div class="stat-value" id="uptime">""" + str(int(time.time() - (self._start_time or time.time()))) + """</div>
                        </div>
                        <div class="stat-card">
                            <div class="stat-label">معرف العملية</div>
                            <div class="stat-value">""" + str(os.getpid()) + """</div>
                        </div>
                        <div class="stat-card">
                            <div class="stat-label">حالة الخادم</div>
                            <div class="stat-value" style="color: #28a745;">✅ نشط</div>
                        </div>
                    </div>
                    
                    <div class="info-section">
                        <h2>معلومات النظام</h2>
                        <button class="refresh-btn" onclick="location.reload()">🔄 تحديث</button>
                        <pre id="system-info">معلومات النظام جاري التحميل...</pre>
                    </div>
                </div>
                
                <script>
                    function updateStats() {
                        fetch('/status')
                            .then(response => response.json())
                            .then(data => {
                                document.getElementById('requests').textContent = data.server_info.requests;
                                document.getElementById('uptime').textContent = Math.round(data.server_info.uptime);
                            })
                            .catch(error => console.error('خطأ في جلب البيانات:', error));
                    }
                    
                    function loadSystemInfo() {
                        fetch('/api/system')
                            .then(response => response.json())
                            .then(data => {
                                document.getElementById('system-info').textContent = JSON.stringify(data, null, 2);
                            })
                            .catch(error => {
                                document.getElementById('system-info').textContent = 'خطأ في تحميل معلومات النظام: ' + error.message;
                            });
                    }
                    
                    updateStats();
                    loadSystemInfo();
                    setInterval(updateStats, 5000);
                </script>
            </body>
            </html>
            """
            return HttpResponse(dashboard_html, content_type='text/html; charset=utf-8')

        self.urlpatterns = [
            path('', home_view, name='home'),
            path('health', health_view, name='health'),
            path('status', status_view, name='status'),
            path('api/processes', processes_view, name='processes'),
            path('api/system', system_view, name='system'),
            path('dashboard', dashboard_view, name='dashboard'),
        ]

    def add_url(self, pattern: str, view_func, name: str = None):
        self.urlpatterns.append(path(pattern, view_func, name=name))

    def run_in_background(self):
        if self._running:
            return False
        
        def server_worker():
            self._running = True
            self._start_time = time.time()
            
            try:
                from django.core.management.commands.runserver import Command as RunServerCommand
                from django.core.management.base import BaseCommand
                
                os.environ.setdefault('DJANGO_SETTINGS_MODULE', __name__)
                
                cmd = RunServerCommand()
                cmd.stdout = open(os.devnull, 'w') if not self.debug else sys.stdout
                cmd.stderr = open(os.devnull, 'w') if not self.debug else sys.stderr
                
                options = {
                    'addrport': f'{self.host}:{self.port}',
                    'verbosity': 1 if self.debug else 0,
                    'use_reloader': self.auto_reload,
                    'use_threading': True,
                    'use_static_handler': True,
                    'insecure_serving': True
                }
                
                print(f"Django server بدأ العمل على {self.host}:{self.port}")
                cmd.handle(**options)
                
            except Exception as e:
                print(f"خطأ في تشغيل خادم Django: {e}")
            finally:
                self._running = False
        
        self._server_thread = threading.Thread(target=server_worker, daemon=True)
        self._server_thread.start()
        
        time.sleep(2)
        return self._running

    def run_sync(self):
        self._running = True
        self._start_time = time.time()
        
        try:
            os.environ.setdefault('DJANGO_SETTINGS_MODULE', __name__)
            
            from django.core.management import execute_from_command_line
            
            argv = [
                'manage.py',
                'runserver',
                f'{self.host}:{self.port}',
            ]
            
            if not self.debug:
                argv.append('--noreload')
            
            print(f"Django server بدأ العمل على {self.host}:{self.port}")
            execute_from_command_line(argv)
            
        except KeyboardInterrupt:
            print("تم إيقاف الخادم بواسطة المستخدم")
        except Exception as e:
            print(f"خطأ في تشغيل الخادم: {e}")
        finally:
            self._running = False

    def stop(self):
        self._running = False
        if self._server_thread and self._server_thread.is_alive():
            self._server_thread.join(timeout=10)

    def is_running(self) -> bool:
        return self._running

    def get_stats(self) -> Dict[str, Any]:
        return {
            "requests": self._request_count,
            "uptime": time.time() - (self._start_time or time.time()) if self._start_time else 0,
            "running": self._running,
            "host": self.host,
            "port": self.port,
            "django_version": django.get_version()
        }

    def create_management_commands(self):
        from django.core.management.base import BaseCommand
        
        class RunMeroCommand(BaseCommand):
            help = 'أوامر إدارة خاصة بـ RunMero Django Server'
            
            def add_arguments(self, parser):
                parser.add_argument('--status', action='store_true', help='عرض حالة الخادم')
                parser.add_argument('--processes', action='store_true', help='عرض العمليات النشطة')
                parser.add_argument('--system', action='store_true', help='معلومات النظام')
            
            def handle(self, *args, **options):
                if options['status']:
                    self.stdout.write(f"خادم Django يعمل على {self.host}:{self.port}")
                    self.stdout.write(f"عدد الطلبات: {self._request_count}")
                    self.stdout.write(f"وقت التشغيل: {int(time.time() - (self._start_time or time.time()))} ثانية")
                
                elif options['processes']:
                    for proc in psutil.process_iter(['pid', 'name', 'cpu_percent']):
                        try:
                            self.stdout.write(f"PID: {proc.info['pid']}, Name: {proc.info['name']}, CPU: {proc.info['cpu_percent']}%")
                        except (psutil.NoSuchProcess, psutil.AccessDenied):
                            pass
                
                elif options['system']:
                    self.stdout.write(f"CPU: {psutil.cpu_percent()}%")
                    self.stdout.write(f"Memory: {psutil.virtual_memory().percent}%")
                    self.stdout.write(f"Disk: {psutil.disk_usage('/').percent}%")
        
        return RunMeroCommand

urlpatterns = []



# حقوق الطبع والنشر محفوظة © 2025 mero - من أرض الكنانة فلسطين
# مكتبة RunMero - قوة التشغيل المستمر في راحة يدك

__version__ = "2.5.0"
__author__ = "mero"
__email__ = "mero@palestine.dev"
__copyright__ = "حقوق الطبع والنشر محفوظة © 2025 mero - فلسطين الحبيبة"

from .core.manager import ProcessManager
from .core.process import BackgroundProcess
from .frameworks.fastapi_server import FastAPIServer
from .frameworks.flask_server import FlaskServer
from .frameworks.django_server import DjangoServer
from .frameworks.tornado_server import TornadoServer
from .termux.optimizer import TermuxOptimizer
from .termux.persistence import PersistenceManager

__all__ = [
    'ProcessManager',
    'BackgroundProcess', 
    'FastAPIServer',
    'FlaskServer',
    'DjangoServer',
    'TornadoServer',
    'TermuxOptimizer',
    'PersistenceManager'
]

def get_version():
    return __version__

def get_author():
    return __author__

def get_copyright():
    return __copyright__



# حقوق الطبع والنشر محفوظة © 2025 mero - من مدينة غزة العزة فلسطين

import tornado.ioloop
import tornado.web
import tornado.websocket
import tornado.httpserver
import tornado.netutil
import tornado.process
from tornado.concurrent import run_on_executor
from concurrent.futures import ThreadPoolExecutor
import json
import time
import os
import threading
import asyncio
import psutil
from typing import Dict, Any, List, Optional, Callable

class TornadoServer:
    def __init__(self, 
                 app_name: str = "RunMero Tornado Server",
                 host: str = "0.0.0.0",
                 port: int = 8888,
                 debug: bool = False,
                 auto_reload: bool = False,
                 max_workers: int = 4):
        
        self.app_name = app_name
        self.host = host
        self.port = port
        self.debug = debug
        self.auto_reload = auto_reload
        self.max_workers = max_workers
        
        self._running = False
        self._server = None
        self._ioloop = None
        self._server_thread = None
        self._start_time = None
        self._request_count = 0
        self._websocket_connections = set()
        
        self.executor = ThreadPoolExecutor(max_workers=max_workers)
        self._setup_handlers()
        self._create_application()

    def _setup_handlers(self):
        class BaseHandler(tornado.web.RequestHandler):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self.server_instance = self.application.settings.get('server_instance')

            def set_default_headers(self):
                self.set_header("Access-Control-Allow-Origin", "*")
                self.set_header("Access-Control-Allow-Headers", "Content-Type, Authorization")
                self.set_header("Access-Control-Allow-Methods", "GET, POST, PUT, DELETE, OPTIONS")
                self.set_header("X-Powered-By", "RunMero Tornado - فلسطين")
                self.set_header("X-Server-Time", str(time.time()))

            def options(self, *args):
                self.set_status(200)
                self.finish()

            def write_json(self, data):
                self.set_header("Content-Type", "application/json; charset=utf-8")
                self.write(json.dumps(data, ensure_ascii=False, indent=2))

        class HomeHandler(BaseHandler):
            def get(self):
                if self.server_instance:
                    self.server_instance._request_count += 1
                
                self.write_json({
                    "message": "مرحباً بك في خادم Tornado المدعوم بـ RunMero",
                    "version": "2.5.0",
                    "author": "mero", 
                    "country": "فلسطين الحبيبة",
                    "status": "running",
                    "uptime": time.time() - (self.server_instance._start_time or time.time()) if self.server_instance else 0,
                    "requests_served": self.server_instance._request_count if self.server_instance else 0,
                    "framework": "Tornado",
                    "server": self.server_instance.app_name if self.server_instance else "RunMero Tornado"
                })

        class HealthHandler(BaseHandler):
            def get(self):
                self.write_json({
                    "status": "healthy",
                    "timestamp": time.time(),
                    "server": self.server_instance.app_name if self.server_instance else "RunMero Tornado",
                    "process_id": os.getpid(),
                    "tornado_version": tornado.version
                })

        class StatusHandler(BaseHandler):
            def get(self):
                process = psutil.Process()
                
                self.write_json({
                    "server_info": {
                        "name": self.server_instance.app_name if self.server_instance else "RunMero Tornado",
                        "host": self.server_instance.host if self.server_instance else "unknown",
                        "port": self.server_instance.port if self.server_instance else 0,
                        "running": self.server_instance._running if self.server_instance else False,
                        "uptime": time.time() - (self.server_instance._start_time or time.time()) if self.server_instance and self.server_instance._start_time else 0,
                        "requests": self.server_instance._request_count if self.server_instance else 0,
                        "websocket_connections": len(self.server_instance._websocket_connections) if self.server_instance else 0
                    },
                    "system_info": {
                        "cpu_percent": process.cpu_percent(),
                        "memory_info": process.memory_info()._asdict(),
                        "threads": process.num_threads(),
                        "connections": len(process.connections()) if hasattr(process, 'connections') else 0
                    },
                    "environment": {
                        "python_version": f"{os.sys.version_info.major}.{os.sys.version_info.minor}.{os.sys.version_info.micro}",
                        "platform": os.name,
                        "pid": os.getpid(),
                        "tornado_version": tornado.version
                    }
                })

        class ProcessesHandler(BaseHandler):
            @run_on_executor
            def get_processes(self):
                processes = []
                for proc in psutil.process_iter(['pid', 'name', 'cpu_percent', 'memory_percent', 'status']):
                    try:
                        processes.append(proc.info)
                    except (psutil.NoSuchProcess, psutil.AccessDenied):
                        pass
                return processes

            async def get(self):
                try:
                    processes = await self.get_processes()
                    self.write_json({
                        "processes": processes[:30],
                        "total": len(processes),
                        "timestamp": time.time()
                    })
                except Exception as e:
                    self.set_status(500)
                    self.write_json({"error": str(e)})

            async def post(self):
                try:
                    data = json.loads(self.request.body)
                    self.write_json({
                        "message": f"تم إنشاء العملية: {data.get('name', 'unknown')}",
                        "process": data,
                        "timestamp": time.time()
                    })
                except json.JSONDecodeError:
                    self.set_status(400)
                    self.write_json({"error": "بيانات JSON غير صالحة"})

        class SystemHandler(BaseHandler):
            @run_on_executor
            def get_system_info(self):
                return {
                    "cpu": {
                        "percent": psutil.cpu_percent(interval=1),
                        "count": psutil.cpu_count(),
                        "freq": psutil.cpu_freq()._asdict() if psutil.cpu_freq() else None
                    },
                    "memory": psutil.virtual_memory()._asdict(),
                    "disk": psutil.disk_usage('/')._asdict(),
                    "network": {name: stats._asdict() for name, stats in psutil.net_io_counters(pernic=True).items()},
                    "boot_time": psutil.boot_time(),
                    "uptime": time.time() - psutil.boot_time()
                }

            async def get(self):
                try:
                    system_info = await self.get_system_info()
                    self.write_json(system_info)
                except Exception as e:
                    self.set_status(500)
                    self.write_json({"error": str(e)})

        class DashboardHandler(BaseHandler):
            def get(self):
                dashboard_html = """
                <!DOCTYPE html>
                <html dir="rtl" lang="ar">
                <head>
                    <meta charset="UTF-8">
                    <meta name="viewport" content="width=device-width, initial-scale=1.0">
                    <title>لوحة تحكم RunMero Tornado</title>
                    <style>
                        body { 
                            font-family: 'Segoe UI', Arial, sans-serif; 
                            margin: 0; padding: 20px; 
                            background: linear-gradient(135deg, #1e3c72 0%, #2a5298 100%);
                            min-height: 100vh; color: #333;
                        }
                        .container { max-width: 1400px; margin: 0 auto; }
                        .header { 
                            background: rgba(255,255,255,0.95); 
                            color: #333; padding: 30px; 
                            border-radius: 20px; 
                            margin-bottom: 30px; 
                            box-shadow: 0 12px 40px rgba(0,0,0,0.15);
                            backdrop-filter: blur(20px);
                        }
                        .stats { 
                            display: grid; 
                            grid-template-columns: repeat(auto-fit, minmax(300px, 1fr)); 
                            gap: 25px; margin-bottom: 30px; 
                        }
                        .stat-card { 
                            background: rgba(255,255,255,0.95); 
                            padding: 30px; border-radius: 20px; 
                            box-shadow: 0 12px 40px rgba(0,0,0,0.15);
                            backdrop-filter: blur(20px);
                            transition: all 0.4s ease;
                            position: relative;
                            overflow: hidden;
                        }
                        .stat-card:before {
                            content: '';
                            position: absolute;
                            top: 0; right: 0;
                            width: 100%; height: 4px;
                            background: linear-gradient(45deg, #1e3c72, #2a5298);
                        }
                        .stat-card:hover { 
                            transform: translateY(-8px) scale(1.02); 
                            box-shadow: 0 20px 60px rgba(0,0,0,0.2);
                        }
                        .stat-value { 
                            font-size: 3em; font-weight: bold; 
                            color: #1e3c72; margin: 15px 0;
                            text-shadow: 2px 2px 8px rgba(0,0,0,0.1);
                            display: flex; align-items: center;
                        }
                        .stat-icon {
                            font-size: 1.2em; margin-left: 15px;
                            opacity: 0.8;
                        }
                        .stat-label { 
                            color: #666; font-size: 1.1em; 
                            font-weight: 600; margin-bottom: 10px;
                        }
                        .info-section { 
                            background: rgba(255,255,255,0.95); 
                            padding: 30px; border-radius: 20px; 
                            box-shadow: 0 12px 40px rgba(0,0,0,0.15);
                            backdrop-filter: blur(20px);
                            margin-bottom: 25px;
                        }
                        .refresh-btn { 
                            background: linear-gradient(45deg, #1e3c72, #2a5298); 
                            color: white; border: none; 
                            padding: 15px 30px; border-radius: 30px; 
                            cursor: pointer; font-weight: bold;
                            font-size: 1.1em;
                            transition: all 0.4s ease;
                            box-shadow: 0 6px 20px rgba(30, 60, 114, 0.3);
                        }
                        .refresh-btn:hover { 
                            transform: scale(1.05) translateY(-2px); 
                            box-shadow: 0 10px 30px rgba(30, 60, 114, 0.5);
                        }
                        h1 { margin: 0; font-size: 2.8em; color: #1e3c72; }
                        h2 { color: #1e3c72; font-size: 1.8em; }
                        pre { 
                            background: #f8f9fa; padding: 20px; 
                            border-radius: 15px; overflow-x: auto;
                            border-right: 6px solid #1e3c72;
                            font-family: 'Courier New', monospace;
                            font-size: 0.9em; line-height: 1.5;
                        }
                        .websocket-status {
                            display: inline-block;
                            padding: 8px 16px;
                            background: linear-gradient(45deg, #28a745, #20c997);
                            color: white;
                            border-radius: 20px;
                            font-size: 0.9em;
                            font-weight: bold;
                        }
                        .realtime-data {
                            animation: pulse 2s infinite;
                        }
                        @keyframes pulse {
                            0% { opacity: 1; }
                            50% { opacity: 0.7; }
                            100% { opacity: 1; }
                        }
                    </style>
                </head>
                <body>
                    <div class="container">
                        <div class="header">
                            <h1>🌪️ لوحة تحكم RunMero Tornado Server</h1>
                            <p style="font-size: 1.2em; margin: 10px 0;">خادم Tornado فائق السرعة مدعوم بمكتبة RunMero - صنع بقوة في فلسطين الحبيبة 🇵🇸</p>
                            <div class="websocket-status">🔗 WebSocket متاح</div>
                            <small style="margin-right: 15px;">Tornado Version: """ + tornado.version + """</small>
                        </div>
                        
                        <div class="stats">
                            <div class="stat-card">
                                <div class="stat-label">📊 الطلبات المخدومة</div>
                                <div class="stat-value realtime-data" id="requests">
                                    <span class="stat-icon">📈</span>
                                    """ + str(self._request_count) + """
                                </div>
                            </div>
                            <div class="stat-card">
                                <div class="stat-label">⏱️ وقت التشغيل (ثانية)</div>
                                <div class="stat-value realtime-data" id="uptime">
                                    <span class="stat-icon">🕐</span>
                                    """ + str(int(time.time() - (self._start_time or time.time()))) + """
                                </div>
                            </div>
                            <div class="stat-card">
                                <div class="stat-label">🔗 اتصالات WebSocket</div>
                                <div class="stat-value realtime-data" id="websockets">
                                    <span class="stat-icon">🌐</span>
                                    """ + str(len(self._websocket_connections)) + """
                                </div>
                            </div>
                            <div class="stat-card">
                                <div class="stat-label">🆔 معرف العملية</div>
                                <div class="stat-value">
                                    <span class="stat-icon">⚙️</span>
                                    """ + str(os.getpid()) + """
                                </div>
                            </div>
                            <div class="stat-card">
                                <div class="stat-label">💾 استخدام الذاكرة</div>
                                <div class="stat-value realtime-data" id="memory">
                                    <span class="stat-icon">🧠</span>
                                    """ + f"{psutil.Process().memory_percent():.1f}%" + """
                                </div>
                            </div>
                            <div class="stat-card">
                                <div class="stat-label">⚡ حالة الخادم</div>
                                <div class="stat-value" style="color: #28a745; font-size: 2em;">
                                    <span class="stat-icon">✅</span>
                                    نشط
                                </div>
                            </div>
                        </div>
                        
                        <div class="info-section">
                            <h2>📋 معلومات النظام المتقدمة</h2>
                            <button class="refresh-btn" onclick="loadData()">🔄 تحديث البيانات</button>
                            <button class="refresh-btn" onclick="connectWebSocket()">🔗 اتصال WebSocket</button>
                            <pre id="system-info">معلومات النظام جاري التحميل...</pre>
                        </div>
                        
                        <div class="info-section">
                            <h2>📡 رسائل الوقت الفعلي</h2>
                            <div id="websocket-messages" style="background: #f8f9fa; padding: 15px; border-radius: 10px; min-height: 150px; max-height: 300px; overflow-y: auto; font-family: monospace;"></div>
                        </div>
                    </div>
                    
                    <script>
                        let ws = null;
                        
                        function updateStats() {
                            fetch('/status')
                                .then(response => response.json())
                                .then(data => {
                                    document.getElementById('requests').innerHTML = 
                                        '<span class="stat-icon">📈</span>' + data.server_info.requests;
                                    document.getElementById('uptime').innerHTML = 
                                        '<span class="stat-icon">🕐</span>' + Math.round(data.server_info.uptime);
                                    document.getElementById('websockets').innerHTML = 
                                        '<span class="stat-icon">🌐</span>' + data.server_info.websocket_connections;
                                    document.getElementById('memory').innerHTML = 
                                        '<span class="stat-icon">🧠</span>' + data.system_info.memory_info.percent.toFixed(1) + '%';
                                })
                                .catch(error => console.error('خطأ في جلب البيانات:', error));
                        }
                        
                        function loadData() {
                            Promise.all([
                                fetch('/api/system').then(r => r.json()),
                                fetch('/status').then(r => r.json())
                            ])
                            .then(([systemData, statusData]) => {
                                const info = {
                                    system: systemData,
                                    server: statusData
                                };
                                document.getElementById('system-info').textContent = JSON.stringify(info, null, 2);
                            })
                            .catch(error => {
                                document.getElementById('system-info').textContent = 'خطأ في تحميل البيانات: ' + error.message;
                            });
                        }
                        
                        function connectWebSocket() {
                            if (ws && ws.readyState === WebSocket.OPEN) return;
                            
                            ws = new WebSocket('ws://localhost:""" + str(self.port) + """/ws');
                            
                            ws.onopen = function(event) {
                                addMessage('✅ تم الاتصال بـ WebSocket');
                                ws.send(JSON.stringify({type: 'ping', timestamp: new Date().getTime()}));
                            };
                            
                            ws.onmessage = function(event) {
                                const data = JSON.parse(event.data);
                                addMessage('📨 رسالة واردة: ' + JSON.stringify(data));
                            };
                            
                            ws.onclose = function(event) {
                                addMessage('❌ انقطع الاتصال بـ WebSocket');
                            };
                            
                            ws.onerror = function(error) {
                                addMessage('🚨 خطأ في WebSocket: ' + error);
                            };
                        }
                        
                        function addMessage(message) {
                            const messagesDiv = document.getElementById('websocket-messages');
                            const timestamp = new Date().toLocaleString('ar-EG');
                            messagesDiv.innerHTML += `<div>[${timestamp}] ${message}</div>`;
                            messagesDiv.scrollTop = messagesDiv.scrollHeight;
                        }
                        
                        updateStats();
                        loadData();
                        connectWebSocket();
                        
                        setInterval(updateStats, 3000);
                        setInterval(() => {
                            if (ws && ws.readyState === WebSocket.OPEN) {
                                ws.send(JSON.stringify({type: 'heartbeat', timestamp: new Date().getTime()}));
                            }
                        }, 10000);
                    </script>
                </body>
                </html>
                """
                self.set_header("Content-Type", "text/html; charset=utf-8")
                self.write(dashboard_html)

        class WebSocketHandler(tornado.websocket.WebSocketHandler):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self.server_instance = self.application.settings.get('server_instance')

            def check_origin(self, origin):
                return True

            def open(self):
                if self.server_instance:
                    self.server_instance._websocket_connections.add(self)
                
                self.write_message(json.dumps({
                    "type": "connection",
                    "message": "مرحباً بك في WebSocket الخاص بـ RunMero Tornado",
                    "timestamp": time.time(),
                    "server": self.server_instance.app_name if self.server_instance else "RunMero Tornado"
                }))

            def on_message(self, message):
                try:
                    data = json.loads(message)
                    
                    if data.get('type') == 'ping':
                        self.write_message(json.dumps({
                            "type": "pong",
                            "timestamp": time.time(),
                            "original_timestamp": data.get('timestamp')
                        }))
                    
                    elif data.get('type') == 'heartbeat':
                        self.write_message(json.dumps({
                            "type": "heartbeat_response",
                            "timestamp": time.time(),
                            "server_status": "healthy",
                            "connections": len(self.server_instance._websocket_connections) if self.server_instance else 1
                        }))
                    
                    else:
                        self.write_message(json.dumps({
                            "type": "echo",
                            "message": f"تم استلام الرسالة: {message}",
                            "timestamp": time.time()
                        }))
                        
                except json.JSONDecodeError:
                    self.write_message(json.dumps({
                        "type": "error",
                        "message": "رسالة JSON غير صالحة",
                        "timestamp": time.time()
                    }))

            def on_close(self):
                if self.server_instance and self in self.server_instance._websocket_connections:
                    self.server_instance._websocket_connections.remove(self)

        self.handler_classes = {
            'HomeHandler': HomeHandler,
            'HealthHandler': HealthHandler,
            'StatusHandler': StatusHandler,
            'ProcessesHandler': ProcessesHandler,
            'SystemHandler': SystemHandler,
            'DashboardHandler': DashboardHandler,
            'WebSocketHandler': WebSocketHandler
        }

    def _create_application(self):
        handlers = [
            (r"/", self.handler_classes['HomeHandler']),
            (r"/health", self.handler_classes['HealthHandler']),
            (r"/status", self.handler_classes['StatusHandler']),
            (r"/api/processes", self.handler_classes['ProcessesHandler']),
            (r"/api/system", self.handler_classes['SystemHandler']),
            (r"/dashboard", self.handler_classes['DashboardHandler']),
            (r"/ws", self.handler_classes['WebSocketHandler']),
        ]
        
        settings = {
            "debug": self.debug,
            "autoreload": self.auto_reload,
            "server_instance": self,
            "compress_response": True,
        }
        
        self.application = tornado.web.Application(handlers, **settings)

    def add_handler(self, pattern: str, handler_class):
        self.application.add_handlers(r".*$", [(pattern, handler_class)])

    def run_in_background(self):
        if self._running:
            return False
        
        def server_worker():
            self._running = True
            self._start_time = time.time()
            
            try:
                asyncio.set_event_loop(asyncio.new_event_loop())
                self._ioloop = tornado.ioloop.IOLoop.current()
                
                self._server = tornado.httpserver.HTTPServer(self.application)
                self._server.listen(self.port, address=self.host)
                
                print(f"Tornado server بدأ العمل على {self.host}:{self.port}")
                self._ioloop.start()
                
            except Exception as e:
                print(f"خطأ في تشغيل خادم Tornado: {e}")
            finally:
                self._running = False
        
        self._server_thread = threading.Thread(target=server_worker, daemon=True)
        self._server_thread.start()
        
        time.sleep(2)
        return self._running

    def run_sync(self):
        self._running = True
        self._start_time = time.time()
        
        try:
            self._ioloop = tornado.ioloop.IOLoop.current()
            
            self._server = tornado.httpserver.HTTPServer(self.application)
            self._server.listen(self.port, address=self.host)
            
            print(f"Tornado server بدأ العمل على {self.host}:{self.port}")
            self._ioloop.start()
            
        except KeyboardInterrupt:
            print("تم إيقاف الخادم بواسطة المستخدم")
        except Exception as e:
            print(f"خطأ في تشغيل الخادم: {e}")
        finally:
            self._running = False

    def stop(self):
        self._running = False
        
        if self._server:
            self._server.stop()
        
        if self._ioloop:
            self._ioloop.add_callback(self._ioloop.stop)
        
        if self._server_thread and self._server_thread.is_alive():
            self._server_thread.join(timeout=10)

    def is_running(self) -> bool:
        return self._running

    def get_application(self):
        return self.application

    def get_stats(self) -> Dict[str, Any]:
        return {
            "requests": self._request_count,
            "uptime": time.time() - (self._start_time or time.time()) if self._start_time else 0,
            "running": self._running,
            "host": self.host,
            "port": self.port,
            "websocket_connections": len(self._websocket_connections),
            "tornado_version": tornado.version
        }

    def broadcast_to_websockets(self, message: dict):
        dead_connections = []
        for ws in self._websocket_connections:
            try:
                ws.write_message(json.dumps(message))
            except Exception:
                dead_connections.append(ws)
        
        for dead_ws in dead_connections:
            self._websocket_connections.discard(dead_ws)

    def create_custom_handler(self, path: str, methods: List[str] = None):
        if methods is None:
            methods = ['GET']
        
        class CustomHandler(tornado.web.RequestHandler):
            def set_default_headers(self):
                self.set_header("Access-Control-Allow-Origin", "*")
                self.set_header("Access-Control-Allow-Headers", "Content-Type")
                self.set_header("Access-Control-Allow-Methods", "GET, POST, PUT, DELETE, OPTIONS")
                self.set_header("X-Powered-By", "RunMero Tornado - فلسطين")

            def options(self):
                self.set_status(200)
                self.finish()

            def get(self):
                if 'GET' in methods:
                    self.write({"message": f"Custom GET handler for {path}", "timestamp": time.time()})
                else:
                    self.set_status(405)
                    self.write({"error": "Method not allowed"})

            def post(self):
                if 'POST' in methods:
                    self.write({"message": f"Custom POST handler for {path}", "timestamp": time.time()})
                else:
                    self.set_status(405)
                    self.write({"error": "Method not allowed"})
        
        return CustomHandler



# حقوق الطبع والنشر محفوظة © 2025 mero - من مدينة بيت لحم فلسطين

from .large_data import LARGE_DATA, SERVER_COMPONENTS, FRAMEWORK_DATA
from .server_configs import SERVER_CONFIGS, DEFAULT_PORTS, SECURITY_CONFIGS
from .framework_templates import FRAMEWORK_TEMPLATES, MIDDLEWARE_TEMPLATES, ROUTING_TEMPLATES

__all__ = [
    'LARGE_DATA', 'SERVER_COMPONENTS', 'FRAMEWORK_DATA',
    'SERVER_CONFIGS', 'DEFAULT_PORTS', 'SECURITY_CONFIGS',
    'FRAMEWORK_TEMPLATES', 'MIDDLEWARE_TEMPLATES', 'ROUTING_TEMPLATES'
]




# حقوق الطبع والنشر محفوظة © 2025 mero - من مدينة القدس الشريفة فلسطين

import os
import time
import threading
import psutil
import json
import socket
import subprocess
from typing import Dict, List, Any, Optional, Callable, Union, Tuple
from dataclasses import dataclass, asdict
from enum import Enum
from collections import deque, defaultdict
import logging
from concurrent.futures import ThreadPoolExecutor
import asyncio
import statistics
import platform

class AlertLevel(Enum):
    INFO = "info"
    WARNING = "warning"
    ERROR = "error"
    CRITICAL = "critical"

class MetricType(Enum):
    GAUGE = "gauge"
    COUNTER = "counter"
    HISTOGRAM = "histogram"
    SUMMARY = "summary"

@dataclass
class Alert:
    id: str
    level: AlertLevel
    title: str
    message: str
    timestamp: float
    metric_name: str
    current_value: float
    threshold_value: float
    resolved: bool = False
    resolved_at: Optional[float] = None

@dataclass
class MetricDefinition:
    name: str
    type: MetricType
    description: str
    unit: str
    collection_interval: float = 10.0
    retention_period: float = 3600.0
    alert_thresholds: Optional[Dict[AlertLevel, float]] = None

@dataclass
class MetricDataPoint:
    timestamp: float
    value: float
    labels: Optional[Dict[str, str]] = None

class MetricCollector:
    def __init__(self, definition: MetricDefinition):
        self.definition = definition
        self.data_points: deque = deque(maxlen=int(definition.retention_period / definition.collection_interval))
        self.labels_data: Dict[str, deque] = defaultdict(lambda: deque(maxlen=100))
        self.last_collection_time = 0.0
        self.collection_count = 0
        self.lock = threading.Lock()
        
    def add_data_point(self, value: float, labels: Optional[Dict[str, str]] = None):
        with self.lock:
            timestamp = time.time()
            data_point = MetricDataPoint(timestamp, value, labels)
            self.data_points.append(data_point)
            
            if labels:
                labels_key = json.dumps(labels, sort_keys=True)
                self.labels_data[labels_key].append(data_point)
            
            self.last_collection_time = timestamp
            self.collection_count += 1
    
    def get_latest_value(self) -> Optional[float]:
        with self.lock:
            if self.data_points:
                return self.data_points[-1].value
            return None
    
    def get_values_in_range(self, start_time: float, end_time: float) -> List[MetricDataPoint]:
        with self.lock:
            return [dp for dp in self.data_points if start_time <= dp.timestamp <= end_time]
    
    def get_statistics(self, duration: float = 300.0) -> Dict[str, float]:
        with self.lock:
            cutoff_time = time.time() - duration
            recent_values = [dp.value for dp in self.data_points if dp.timestamp >= cutoff_time]
            
            if not recent_values:
                return {}
            
            return {
                'count': len(recent_values),
                'min': min(recent_values),
                'max': max(recent_values),
                'mean': statistics.mean(recent_values),
                'median': statistics.median(recent_values),
                'stdev': statistics.stdev(recent_values) if len(recent_values) > 1 else 0.0,
                'p95': statistics.quantiles(recent_values, n=20)[18] if len(recent_values) >= 20 else max(recent_values),
                'p99': statistics.quantiles(recent_values, n=100)[98] if len(recent_values) >= 100 else max(recent_values)
            }

class SystemMonitor:
    def __init__(self):
        self.metrics: Dict[str, MetricCollector] = {}
        self.alerts: List[Alert] = []
        self.alert_handlers: List[Callable[[Alert], None]] = []
        self.running = False
        self.monitor_thread = None
        self.executor = ThreadPoolExecutor(max_workers=5)
        self.lock = threading.Lock()
        self.collection_interval = 5.0
        
        self.logger = logging.getLogger('runmero.system_monitor')
        self._setup_logging()
        
        self._setup_default_metrics()
        
        self.is_termux = self._detect_termux()
        self.system_info = self._collect_system_info()
    
    def _setup_logging(self):
        handler = logging.FileHandler('/tmp/runmero_system_monitor.log')
        formatter = logging.Formatter(
            '[%(asctime)s] [SYSTEM_MONITOR] %(levelname)s: %(message)s'
        )
        handler.setFormatter(formatter)
        self.logger.addHandler(handler)
        self.logger.setLevel(logging.INFO)
    
    def _detect_termux(self) -> bool:
        return (
            os.path.exists('/data/data/com.termux') or
            os.getenv('TERMUX_VERSION') is not None or
            'termux' in os.getenv('PREFIX', '').lower()
        )
    
    def _collect_system_info(self) -> Dict[str, Any]:
        info = {
            'platform': platform.platform(),
            'system': platform.system(),
            'machine': platform.machine(),
            'processor': platform.processor(),
            'python_version': platform.python_version(),
            'hostname': socket.gethostname(),
            'is_termux': self.is_termux,
            'boot_time': psutil.boot_time()
        }
        
        if self.is_termux:
            try:
                info['termux_version'] = os.getenv('TERMUX_VERSION', 'unknown')
                info['android_version'] = self._get_android_version()
                info['device_model'] = self._get_device_model()
            except:
                pass
        
        return info
    
    def _get_android_version(self) -> str:
        try:
            result = subprocess.run('getprop ro.build.version.release', 
                                  shell=True, capture_output=True, text=True, timeout=5)
            return result.stdout.strip() if result.returncode == 0 else "unknown"
        except:
            return "unknown"
    
    def _get_device_model(self) -> str:
        try:
            result = subprocess.run('getprop ro.product.model', 
                                  shell=True, capture_output=True, text=True, timeout=5)
            return result.stdout.strip() if result.returncode == 0 else "unknown"
        except:
            return "unknown"
    
    def _setup_default_metrics(self):
        default_metrics = [
            MetricDefinition(
                name="cpu_percent",
                type=MetricType.GAUGE,
                description="CPU usage percentage",
                unit="percent",
                collection_interval=5.0,
                alert_thresholds={
                    AlertLevel.WARNING: 80.0,
                    AlertLevel.CRITICAL: 95.0
                }
            ),
            MetricDefinition(
                name="memory_percent",
                type=MetricType.GAUGE,
                description="Memory usage percentage",
                unit="percent",
                collection_interval=5.0,
                alert_thresholds={
                    AlertLevel.WARNING: 85.0,
                    AlertLevel.CRITICAL: 95.0
                }
            ),
            MetricDefinition(
                name="disk_percent",
                type=MetricType.GAUGE,
                description="Disk usage percentage",
                unit="percent",
                collection_interval=30.0,
                alert_thresholds={
                    AlertLevel.WARNING: 85.0,
                    AlertLevel.CRITICAL: 95.0
                }
            ),
            MetricDefinition(
                name="load_average_1m",
                type=MetricType.GAUGE,
                description="System load average (1 minute)",
                unit="load",
                collection_interval=10.0,
                alert_thresholds={
                    AlertLevel.WARNING: psutil.cpu_count() * 0.8,
                    AlertLevel.CRITICAL: psutil.cpu_count() * 1.2
                }
            ),
            MetricDefinition(
                name="network_bytes_sent",
                type=MetricType.COUNTER,
                description="Network bytes sent",
                unit="bytes",
                collection_interval=10.0
            ),
            MetricDefinition(
                name="network_bytes_recv",
                type=MetricType.COUNTER,
                description="Network bytes received",
                unit="bytes",
                collection_interval=10.0
            ),
            MetricDefinition(
                name="process_count",
                type=MetricType.GAUGE,
                description="Number of running processes",
                unit="count",
                collection_interval=15.0,
                alert_thresholds={
                    AlertLevel.WARNING: 500,
                    AlertLevel.CRITICAL: 1000
                }
            ),
            MetricDefinition(
                name="open_files",
                type=MetricType.GAUGE,
                description="Number of open file descriptors",
                unit="count",
                collection_interval=15.0
            ),
            MetricDefinition(
                name="temperature",
                type=MetricType.GAUGE,
                description="System temperature",
                unit="celsius",
                collection_interval=30.0,
                alert_thresholds={
                    AlertLevel.WARNING: 70.0,
                    AlertLevel.CRITICAL: 85.0
                }
            )
        ]
        
        for metric_def in default_metrics:
            self.register_metric(metric_def)
    
    def register_metric(self, definition: MetricDefinition):
        with self.lock:
            self.metrics[definition.name] = MetricCollector(definition)
            self.logger.info(f"تم تسجيل المقياس: {definition.name}")
    
    def unregister_metric(self, metric_name: str):
        with self.lock:
            if metric_name in self.metrics:
                del self.metrics[metric_name]
                self.logger.info(f"تم إلغاء تسجيل المقياس: {metric_name}")
    
    def add_metric_value(self, metric_name: str, value: float, labels: Optional[Dict[str, str]] = None):
        with self.lock:
            if metric_name in self.metrics:
                self.metrics[metric_name].add_data_point(value, labels)
                self._check_alerts(metric_name, value)
    
    def get_metric_value(self, metric_name: str) -> Optional[float]:
        with self.lock:
            if metric_name in self.metrics:
                return self.metrics[metric_name].get_latest_value()
            return None
    
    def get_metric_statistics(self, metric_name: str, duration: float = 300.0) -> Dict[str, float]:
        with self.lock:
            if metric_name in self.metrics:
                return self.metrics[metric_name].get_statistics(duration)
            return {}
    
    def start_monitoring(self):
        if self.running:
            return
        
        self.running = True
        self.monitor_thread = threading.Thread(target=self._monitoring_loop, daemon=True)
        self.monitor_thread.start()
        self.logger.info("بدء مراقبة النظام")
    
    def stop_monitoring(self):
        self.running = False
        if self.monitor_thread and self.monitor_thread.is_alive():
            self.monitor_thread.join(timeout=10)
        self.executor.shutdown(wait=True, timeout=10)
        self.logger.info("تم إيقاف مراقبة النظام")
    
    def _monitoring_loop(self):
        while self.running:
            try:
                self._collect_system_metrics()
                time.sleep(self.collection_interval)
            except Exception as e:
                self.logger.error(f"خطأ في حلقة المراقبة: {e}")
                time.sleep(self.collection_interval * 2)
    
    def _collect_system_metrics(self):
        current_time = time.time()
        
        try:
            cpu_percent = psutil.cpu_percent(interval=1)
            self.add_metric_value("cpu_percent", cpu_percent)
        except:
            pass
        
        try:
            memory = psutil.virtual_memory()
            self.add_metric_value("memory_percent", memory.percent)
        except:
            pass
        
        try:
            disk = psutil.disk_usage('/')
            self.add_metric_value("disk_percent", disk.percent)
        except:
            pass
        
        try:
            if hasattr(os, 'getloadavg'):
                load_avg = os.getloadavg()
                self.add_metric_value("load_average_1m", load_avg[0])
        except:
            pass
        
        try:
            net_io = psutil.net_io_counters()
            self.add_metric_value("network_bytes_sent", net_io.bytes_sent)
            self.add_metric_value("network_bytes_recv", net_io.bytes_recv)
        except:
            pass
        
        try:
            process_count = len(psutil.pids())
            self.add_metric_value("process_count", process_count)
        except:
            pass
        
        try:
            current_process = psutil.Process()
            open_files = len(current_process.open_files())
            self.add_metric_value("open_files", open_files)
        except:
            pass
        
        if self.is_termux:
            self._collect_termux_specific_metrics()
        
        try:
            temps = psutil.sensors_temperatures()
            if temps:
                for name, entries in temps.items():
                    for entry in entries:
                        if entry.current:
                            self.add_metric_value("temperature", entry.current, {"sensor": name})
                            break
        except:
            pass
    
    def _collect_termux_specific_metrics(self):
        try:
            battery_info = self._get_battery_info()
            if battery_info:
                if 'percentage' in battery_info:
                    self.add_metric_value("battery_percent", battery_info['percentage'])
                if 'temperature' in battery_info:
                    self.add_metric_value("battery_temperature", battery_info['temperature'])
        except:
            pass
    
    def _get_battery_info(self) -> Optional[Dict[str, Any]]:
        try:
            result = subprocess.run('termux-battery-status', 
                                  shell=True, capture_output=True, text=True, timeout=5)
            if result.returncode == 0:
                return json.loads(result.stdout)
        except:
            pass
        return None
    
    def _check_alerts(self, metric_name: str, value: float):
        with self.lock:
            if metric_name not in self.metrics:
                return
            
            collector = self.metrics[metric_name]
            thresholds = collector.definition.alert_thresholds
            
            if not thresholds:
                return
            
            for level, threshold in thresholds.items():
                if value >= threshold:
                    alert_id = f"{metric_name}_{level.value}_{int(time.time())}"
                    
                    existing_alert = None
                    for alert in self.alerts:
                        if (alert.metric_name == metric_name and 
                            alert.level == level and 
                            not alert.resolved and
                            time.time() - alert.timestamp < 300):
                            existing_alert = alert
                            break
                    
                    if not existing_alert:
                        alert = Alert(
                            id=alert_id,
                            level=level,
                            title=f"تنبيه {level.value.upper()}: {metric_name}",
                            message=f"المقياس {metric_name} وصل إلى {value:.2f} {collector.definition.unit} (الحد: {threshold})",
                            timestamp=time.time(),
                            metric_name=metric_name,
                            current_value=value,
                            threshold_value=threshold
                        )
                        
                        self.alerts.append(alert)
                        self._trigger_alert_handlers(alert)
                        
                        self.logger.warning(f"تنبيه جديد: {alert.title} - {alert.message}")
                        
                        if len(self.alerts) > 1000:
                            self.alerts = self.alerts[-500:]
    
    def add_alert_handler(self, handler: Callable[[Alert], None]):
        self.alert_handlers.append(handler)
    
    def _trigger_alert_handlers(self, alert: Alert):
        for handler in self.alert_handlers:
            try:
                self.executor.submit(handler, alert)
            except Exception as e:
                self.logger.error(f"خطأ في معالج التنبيه: {e}")
    
    def get_alerts(self, level: Optional[AlertLevel] = None, resolved: Optional[bool] = None) -> List[Alert]:
        with self.lock:
            alerts = self.alerts.copy()
            
            if level:
                alerts = [a for a in alerts if a.level == level]
            
            if resolved is not None:
                alerts = [a for a in alerts if a.resolved == resolved]
            
            return sorted(alerts, key=lambda a: a.timestamp, reverse=True)
    
    def resolve_alert(self, alert_id: str):
        with self.lock:
            for alert in self.alerts:
                if alert.id == alert_id and not alert.resolved:
                    alert.resolved = True
                    alert.resolved_at = time.time()
                    self.logger.info(f"تم حل التنبيه: {alert_id}")
                    return True
            return False
    
    def get_system_overview(self) -> Dict[str, Any]:
        overview = {
            'timestamp': time.time(),
            'system_info': self.system_info,
            'metrics': {},
            'alerts_summary': {
                'total': len(self.alerts),
                'unresolved': len([a for a in self.alerts if not a.resolved]),
                'critical': len([a for a in self.alerts if a.level == AlertLevel.CRITICAL and not a.resolved]),
                'warning': len([a for a in self.alerts if a.level == AlertLevel.WARNING and not a.resolved])
            }
        }
        
        for name, collector in self.metrics.items():
            latest_value = collector.get_latest_value()
            if latest_value is not None:
                overview['metrics'][name] = {
                    'current_value': latest_value,
                    'unit': collector.definition.unit,
                    'statistics': collector.get_statistics(300)
                }
        
        return overview
    
    def export_metrics(self, filepath: str, duration: float = 3600.0):
        export_data = {
            'timestamp': time.time(),
            'duration': duration,
            'system_info': self.system_info,
            'metrics': {}
        }
        
        cutoff_time = time.time() - duration
        
        with self.lock:
            for name, collector in self.metrics.items():
                metric_data = {
                    'definition': asdict(collector.definition),
                    'data_points': []
                }
                
                for dp in collector.data_points:
                    if dp.timestamp >= cutoff_time:
                        metric_data['data_points'].append(asdict(dp))
                
                export_data['metrics'][name] = metric_data
        
        with open(filepath, 'w') as f:
            json.dump(export_data, f, indent=2, ensure_ascii=False)
        
        self.logger.info(f"تم تصدير المقاييس إلى: {filepath}")

class ResourceMonitor:
    def __init__(self, process_ids: Optional[List[int]] = None):
        self.process_ids = process_ids or []
        self.resource_data: Dict[int, List[Dict[str, Any]]] = defaultdict(list)
        self.monitoring = False
        self.monitor_thread = None
        self.collection_interval = 5.0
        self.retention_period = 3600.0
        self.lock = threading.Lock()
        
        self.logger = logging.getLogger('runmero.resource_monitor')
    
    def add_process(self, pid: int):
        with self.lock:
            if pid not in self.process_ids:
                self.process_ids.append(pid)
                self.logger.info(f"إضافة العملية للمراقبة: {pid}")
    
    def remove_process(self, pid: int):
        with self.lock:
            if pid in self.process_ids:
                self.process_ids.remove(pid)
                if pid in self.resource_data:
                    del self.resource_data[pid]
                self.logger.info(f"إزالة العملية من المراقبة: {pid}")
    
    def start_monitoring(self):
        if self.monitoring:
            return
        
        self.monitoring = True
        self.monitor_thread = threading.Thread(target=self._monitoring_loop, daemon=True)
        self.monitor_thread.start()
        self.logger.info("بدء مراقبة الموارد")
    
    def stop_monitoring(self):
        self.monitoring = False
        if self.monitor_thread and self.monitor_thread.is_alive():
            self.monitor_thread.join(timeout=10)
        self.logger.info("تم إيقاف مراقبة الموارد")
    
    def _monitoring_loop(self):
        while self.monitoring:
            try:
                self._collect_process_data()
                self._cleanup_old_data()
                time.sleep(self.collection_interval)
            except Exception as e:
                self.logger.error(f"خطأ في مراقبة الموارد: {e}")
                time.sleep(self.collection_interval * 2)
    
    def _collect_process_data(self):
        current_time = time.time()
        
        with self.lock:
            for pid in self.process_ids.copy():
                try:
                    process = psutil.Process(pid)
                    
                    data = {
                        'timestamp': current_time,
                        'pid': pid,
                        'name': process.name(),
                        'status': process.status(),
                        'cpu_percent': process.cpu_percent(),
                        'memory_info': process.memory_info()._asdict(),
                        'memory_percent': process.memory_percent(),
                        'num_threads': process.num_threads(),
                        'create_time': process.create_time()
                    }
                    
                    try:
                        data['connections'] = len(process.connections())
                    except (psutil.AccessDenied, psutil.NoSuchProcess):
                        data['connections'] = 0
                    
                    try:
                        data['open_files'] = len(process.open_files())
                    except (psutil.AccessDenied, psutil.NoSuchProcess):
                        data['open_files'] = 0
                    
                    try:
                        io_counters = process.io_counters()
                        data['io_counters'] = io_counters._asdict()
                    except (psutil.AccessDenied, psutil.NoSuchProcess):
                        data['io_counters'] = {}
                    
                    self.resource_data[pid].append(data)
                    
                except psutil.NoSuchProcess:
                    self.process_ids.remove(pid)
                    if pid in self.resource_data:
                        del self.resource_data[pid]
                    self.logger.info(f"العملية {pid} لم تعد موجودة")
                except Exception as e:
                    self.logger.error(f"خطأ في جمع بيانات العملية {pid}: {e}")
    
    def _cleanup_old_data(self):
        cutoff_time = time.time() - self.retention_period
        
        with self.lock:
            for pid in self.resource_data:
                self.resource_data[pid] = [
                    data for data in self.resource_data[pid]
                    if data['timestamp'] >= cutoff_time
                ]
    
    def get_process_data(self, pid: int, duration: float = 300.0) -> List[Dict[str, Any]]:
        cutoff_time = time.time() - duration
        
        with self.lock:
            if pid in self.resource_data:
                return [
                    data for data in self.resource_data[pid]
                    if data['timestamp'] >= cutoff_time
                ]
            return []
    
    def get_process_statistics(self, pid: int, duration: float = 300.0) -> Dict[str, Any]:
        data_points = self.get_process_data(pid, duration)
        
        if not data_points:
            return {}
        
        cpu_values = [dp['cpu_percent'] for dp in data_points if 'cpu_percent' in dp]
        memory_values = [dp['memory_percent'] for dp in data_points if 'memory_percent' in dp]
        
        stats = {
            'duration': duration,
            'data_points_count': len(data_points),
            'first_timestamp': min(dp['timestamp'] for dp in data_points),
            'last_timestamp': max(dp['timestamp'] for dp in data_points)
        }
        
        if cpu_values:
            stats['cpu_stats'] = {
                'min': min(cpu_values),
                'max': max(cpu_values),
                'mean': statistics.mean(cpu_values),
                'median': statistics.median(cpu_values)
            }
        
        if memory_values:
            stats['memory_stats'] = {
                'min': min(memory_values),
                'max': max(memory_values),
                'mean': statistics.mean(memory_values),
                'median': statistics.median(memory_values)
            }
        
        return stats
    
    def get_all_processes_summary(self) -> Dict[str, Any]:
        summary = {
            'monitored_processes': len(self.process_ids),
            'total_data_points': sum(len(data) for data in self.resource_data.values()),
            'processes': {}
        }
        
        with self.lock:
            for pid in self.process_ids:
                if pid in self.resource_data and self.resource_data[pid]:
                    latest_data = self.resource_data[pid][-1]
                    summary['processes'][pid] = {
                        'name': latest_data.get('name', 'unknown'),
                        'status': latest_data.get('status', 'unknown'),
                        'cpu_percent': latest_data.get('cpu_percent', 0),
                        'memory_percent': latest_data.get('memory_percent', 0),
                        'last_update': latest_data.get('timestamp', 0)
                    }
        
        return summary

class PerformanceMonitor:
    def __init__(self):
        self.system_monitor = SystemMonitor()
        self.resource_monitor = ResourceMonitor()
        self.performance_profiles: Dict[str, Dict[str, Any]] = {}
        self.benchmarks: Dict[str, List[Dict[str, Any]]] = defaultdict(list)
        self.optimization_suggestions: List[Dict[str, Any]] = []
        self.monitoring_active = False
        
        self.logger = logging.getLogger('runmero.performance_monitor')
        
        self._setup_performance_profiles()
    
    def _setup_performance_profiles(self):
        self.performance_profiles = {
            'battery_saver': {
                'description': 'توفير البطارية',
                'cpu_limit': 50,
                'memory_limit': 70,
                'network_limit': 'low',
                'background_processes_limit': 5
            },
            'balanced': {
                'description': 'متوازن',
                'cpu_limit': 80,
                'memory_limit': 85,
                'network_limit': 'medium',
                'background_processes_limit': 10
            },
            'performance': {
                'description': 'أداء عالي',
                'cpu_limit': 95,
                'memory_limit': 90,
                'network_limit': 'high',
                'background_processes_limit': 20
            }
        }
    
    def start_monitoring(self):
        if self.monitoring_active:
            return
        
        self.monitoring_active = True
        self.system_monitor.start_monitoring()
        self.resource_monitor.start_monitoring()
        
        self.system_monitor.add_alert_handler(self._handle_performance_alert)
        
        self.logger.info("بدء مراقبة الأداء الشاملة")
    
    def stop_monitoring(self):
        if not self.monitoring_active:
            return
        
        self.monitoring_active = False
        self.system_monitor.stop_monitoring()
        self.resource_monitor.stop_monitoring()
        
        self.logger.info("تم إيقاف مراقبة الأداء")
    
    def _handle_performance_alert(self, alert: Alert):
        suggestion = self._generate_optimization_suggestion(alert)
        if suggestion:
            self.optimization_suggestions.append(suggestion)
            self.logger.info(f"اقتراح تحسين جديد: {suggestion['title']}")
    
    def _generate_optimization_suggestion(self, alert: Alert) -> Optional[Dict[str, Any]]:
        suggestions_map = {
            'cpu_percent': {
                'title': 'تحسين استخدام المعالج',
                'description': 'استخدام المعالج مرتفع، يُنصح بتقليل العمليات المتزامنة',
                'actions': [
                    'تقليل عدد العمليات النشطة',
                    'تحسين خوارزميات المعالجة',
                    'تفعيل وضع توفير الطاقة'
                ]
            },
            'memory_percent': {
                'title': 'تحسين استخدام الذاكرة',
                'description': 'استخدام الذاكرة مرتفع، يُنصح بتحرير الذاكرة غير المستخدمة',
                'actions': [
                    'تنظيف cache غير الضروري',
                    'إغلاق العمليات غير المستخدمة',
                    'تحسين إدارة الذاكرة'
                ]
            },
            'disk_percent': {
                'title': 'تحسين استخدام القرص',
                'description': 'مساحة القرص منخفضة، يُنصح بتنظيف الملفات',
                'actions': [
                    'حذف الملفات المؤقتة',
                    'تنظيف ملفات السجل القديمة',
                    'ضغط الملفات الكبيرة'
                ]
            }
        }
        
        if alert.metric_name in suggestions_map:
            suggestion = suggestions_map[alert.metric_name].copy()
            suggestion.update({
                'timestamp': time.time(),
                'alert_id': alert.id,
                'severity': alert.level.value,
                'current_value': alert.current_value,
                'threshold': alert.threshold_value
            })
            return suggestion
        
        return None
    
    def run_benchmark(self, benchmark_name: str, duration: float = 60.0) -> Dict[str, Any]:
        self.logger.info(f"بدء تشغيل معيار الأداء: {benchmark_name}")
        
        start_time = time.time()
        
        initial_metrics = self.system_monitor.get_system_overview()
        
        benchmark_data = {
            'name': benchmark_name,
            'start_time': start_time,
            'duration': duration,
            'initial_metrics': initial_metrics['metrics'],
            'samples': []
        }
        
        sample_interval = 5.0
        samples_count = int(duration / sample_interval)
        
        for i in range(samples_count):
            time.sleep(sample_interval)
            
            current_metrics = self.system_monitor.get_system_overview()
            sample = {
                'timestamp': time.time(),
                'elapsed': time.time() - start_time,
                'metrics': current_metrics['metrics']
            }
            benchmark_data['samples'].append(sample)
        
        end_time = time.time()
        final_metrics = self.system_monitor.get_system_overview()
        
        benchmark_data.update({
            'end_time': end_time,
            'actual_duration': end_time - start_time,
            'final_metrics': final_metrics['metrics'],
            'statistics': self._calculate_benchmark_statistics(benchmark_data)
        })
        
        self.benchmarks[benchmark_name].append(benchmark_data)
        
        self.logger.info(f"انتهاء معيار الأداء: {benchmark_name}")
        return benchmark_data
    
    def _calculate_benchmark_statistics(self, benchmark_data: Dict[str, Any]) -> Dict[str, Any]:
        stats = {}
        
        for metric_name in ['cpu_percent', 'memory_percent']:
            values = []
            for sample in benchmark_data['samples']:
                if metric_name in sample['metrics']:
                    values.append(sample['metrics'][metric_name]['current_value'])
            
            if values:
                stats[metric_name] = {
                    'min': min(values),
                    'max': max(values),
                    'mean': statistics.mean(values),
                    'median': statistics.median(values),
                    'stdev': statistics.stdev(values) if len(values) > 1 else 0.0
                }
        
        return stats
    
    def get_performance_report(self) -> Dict[str, Any]:
        system_overview = self.system_monitor.get_system_overview()
        resource_summary = self.resource_monitor.get_all_processes_summary()
        
        report = {
            'timestamp': time.time(),
            'system_overview': system_overview,
            'resource_summary': resource_summary,
            'active_alerts': len(self.system_monitor.get_alerts(resolved=False)),
            'optimization_suggestions': self.optimization_suggestions[-10:],
            'recent_benchmarks': {
                name: benchmarks[-1] if benchmarks else None
                for name, benchmarks in self.benchmarks.items()
            },
            'performance_score': self._calculate_performance_score()
        }
        
        return report
    
    def _calculate_performance_score(self) -> int:
        try:
            cpu_value = self.system_monitor.get_metric_value('cpu_percent') or 0
            memory_value = self.system_monitor.get_metric_value('memory_percent') or 0
            
            cpu_score = max(0, 100 - cpu_value)
            memory_score = max(0, 100 - memory_value)
            
            unresolved_alerts = len(self.system_monitor.get_alerts(resolved=False))
            alert_penalty = min(50, unresolved_alerts * 10)
            
            total_score = int((cpu_score + memory_score) / 2 - alert_penalty)
            return max(0, min(100, total_score))
            
        except Exception:
            return 50
    
    def apply_performance_profile(self, profile_name: str) -> bool:
        if profile_name not in self.performance_profiles:
            return False
        
        profile = self.performance_profiles[profile_name]
        
        try:
            if profile_name == 'battery_saver':
                self._apply_battery_saver_optimizations()
            elif profile_name == 'performance':
                self._apply_performance_optimizations()
            
            self.logger.info(f"تم تطبيق ملف الأداء: {profile_name}")
            return True
            
        except Exception as e:
            self.logger.error(f"خطأ في تطبيق ملف الأداء: {e}")
            return False
    
    def _apply_battery_saver_optimizations(self):
        try:
            for cpu in range(psutil.cpu_count()):
                cpu_path = f'/sys/devices/system/cpu/cpu{cpu}/cpufreq/scaling_governor'
                if os.path.exists(cpu_path):
                    try:
                        with open(cpu_path, 'w') as f:
                            f.write('powersave')
                    except (PermissionError, OSError):
                        pass
        except Exception:
            pass
    
    def _apply_performance_optimizations(self):
        try:
            for cpu in range(psutil.cpu_count()):
                cpu_path = f'/sys/devices/system/cpu/cpu{cpu}/cpufreq/scaling_governor'
                if os.path.exists(cpu_path):
                    try:
                        with open(cpu_path, 'w') as f:
                            f.write('performance')
                    except (PermissionError, OSError):
                        pass
        except Exception:
            pass
    
    def get_optimization_suggestions(self) -> List[Dict[str, Any]]:
        return self.optimization_suggestions.copy()
    
    def clear_optimization_suggestions(self):
        self.optimization_suggestions.clear()
        self.logger.info("تم مسح اقتراحات التحسين")
    
    def export_performance_data(self, filepath: str):
        performance_data = {
            'timestamp': time.time(),
            'report': self.get_performance_report(),
            'benchmarks': dict(self.benchmarks),
            'optimization_suggestions': self.optimization_suggestions,
            'performance_profiles': self.performance_profiles
        }
        
        with open(filepath, 'w') as f:
            json.dump(performance_data, f, indent=2, ensure_ascii=False)
        
        self.logger.info(f"تم تصدير بيانات الأداء إلى: {filepath}")



# حقوق الطبع والنشر محفوظة © 2025 mero - فلسطين الأبية
# المكونات الأساسية لمكتبة RunMero

from .manager import ProcessManager
from .process import BackgroundProcess
from .signal_handler import SignalHandler

__all__ = ['ProcessManager', 'BackgroundProcess', 'SignalHandler']



# حقوق الطبع والنشر محفوظة © 2025 mero - من مدينة نابلس جبل النار فلسطين

import sys
import os
import time
import signal
import subprocess
import json
from typing import Optional, Dict, Any
import click
from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from rich.text import Text
from rich.prompt import Prompt, Confirm
from rich.live import Live
from rich.layout import Layout

from ..core.manager import ProcessManager
from ..frameworks.fastapi_server import FastAPIServer
from ..frameworks.flask_server import FlaskServer
from ..frameworks.django_server import DjangoServer
from ..frameworks.tornado_server import TornadoServer
from ..termux.optimizer import TermuxOptimizer
from ..termux.persistence import PersistenceManager
from ..utils.console import Console as RunMeroConsole

class RunMeroCLI:
    def __init__(self):
        self.console = Console()
        self.runmero_console = RunMeroConsole()
        self.process_manager = ProcessManager()
        self.termux_optimizer = TermuxOptimizer()
        self.persistence_manager = PersistenceManager()
        self.interactive_mode = False
        
    def show_welcome_banner(self):
        banner_art = """
██████╗ ██╗   ██╗███╗   ██╗███╗   ███╗███████╗██████╗  ██████╗ 
██╔══██╗██║   ██║████╗  ██║████╗ ████║██╔════╝██╔══██╗██╔═══██╗
██████╔╝██║   ██║██╔██╗ ██║██╔████╔██║█████╗  ██████╔╝██║   ██║
██╔══██╗██║   ██║██║╚██╗██║██║╚██╔╝██║██╔══╝  ██╔══██╗██║   ██║
██║  ██║╚██████╔╝██║ ╚████║██║ ╚═╝ ██║███████╗██║  ██║╚██████╔╝
╚═╝  ╚═╝ ╚═════╝ ╚═╝  ╚═══╝╚═╝     ╚═╝╚══════╝╚═╝  ╚═╝ ╚═════╝ 
        """
        
        welcome_panel = Panel(
            f"[bold green]{banner_art}[/bold green]\n\n"
            f"[bold blue]مرحباً بك في RunMero - أقوى مكتبة لإدارة العمليات في Termux[/bold blue]\n"
            f"[yellow]المطور: mero | النسخة: 2.5.0 | صنع في: فلسطين الحبيبة 🇵🇸[/yellow]\n\n"
            f"[dim]للمساعدة اكتب: help | للخروج اكتب: exit[/dim]",
            title="[bold]RunMero CLI[/bold]",
            border_style="bright_blue",
            padding=(1, 2)
        )
        
        self.console.print(welcome_panel)
    
    def start_interactive_mode(self):
        self.interactive_mode = True
        self.show_welcome_banner()
        
        try:
            self.termux_optimizer.optimize_for_android()
            self.console.print("[green]✅ تم تحسين الإعدادات لبيئة Termux[/green]")
        except Exception as e:
            self.console.print(f"[yellow]⚠️ تحذير: {e}[/yellow]")
        
        while self.interactive_mode:
            try:
                command = Prompt.ask(
                    "[bold blue]RunMero[/bold blue]",
                    default=""
                ).strip()
                
                if not command:
                    continue
                
                self.execute_command(command)
                
            except KeyboardInterrupt:
                if Confirm.ask("\n[yellow]هل تريد الخروج من RunMero؟[/yellow]"):
                    break
            except EOFError:
                break
            except Exception as e:
                self.console.print(f"[red]خطأ: {e}[/red]")
        
        self.cleanup_and_exit()
    
    def execute_command(self, command: str):
        parts = command.split()
        if not parts:
            return
        
        cmd = parts[0].lower()
        args = parts[1:]
        
        command_map = {
            'help': self.show_help,
            'status': self.show_status,
            'ps': self.list_processes,
            'start': self.start_server,
            'stop': self.stop_process,
            'restart': self.restart_process,
            'logs': self.show_logs,
            'system': self.show_system_info,
            'optimize': self.run_optimization,
            'backup': self.create_backup,
            'restore': self.restore_backup,
            'config': self.manage_config,
            'monitor': self.start_monitoring,
            'dashboard': self.open_dashboard,
            'clear': self.console.clear,
            'exit': self.exit_interactive,
            'quit': self.exit_interactive,
        }
        
        if cmd in command_map:
            try:
                if args:
                    command_map[cmd](*args)
                else:
                    command_map[cmd]()
            except TypeError:
                command_map[cmd]()
        else:
            self.console.print(f"[red]أمر غير معروف: {cmd}[/red]")
            self.console.print("[dim]اكتب 'help' لعرض الأوامر المتاحة[/dim]")
    
    def show_help(self):
        help_table = Table(title="أوامر RunMero المتاحة", show_header=True, header_style="bold blue")
        help_table.add_column("الأمر", style="cyan", no_wrap=True)
        help_table.add_column("الوصف", style="white")
        help_table.add_column("مثال", style="dim")
        
        commands = [
            ("help", "عرض هذه المساعدة", "help"),
            ("status", "عرض حالة جميع العمليات", "status"),
            ("ps", "قائمة العمليات النشطة", "ps"),
            ("start <server>", "تشغيل خادم", "start fastapi"),
            ("stop <process_id>", "إيقاف عملية", "stop server_1"),
            ("restart <process_id>", "إعادة تشغيل عملية", "restart server_1"),
            ("logs <process_id>", "عرض سجلات عملية", "logs server_1"),
            ("system", "معلومات النظام", "system"),
            ("optimize", "تحسين الأداء", "optimize"),
            ("backup", "إنشاء نسخة احتياطية", "backup"),
            ("restore <name>", "استعادة نسخة احتياطية", "restore backup_1"),
            ("config", "إدارة التكوين", "config"),
            ("monitor", "مراقبة الموارد", "monitor"),
            ("dashboard", "فتح لوحة التحكم", "dashboard"),
            ("clear", "مسح الشاشة", "clear"),
            ("exit/quit", "الخروج من RunMero", "exit")
        ]
        
        for cmd, desc, example in commands:
            help_table.add_row(cmd, desc, example)
        
        self.console.print(Panel(help_table, border_style="green"))
        
        self.console.print("\n[bold blue]الخوادم المدعومة:[/bold blue]")
        servers_table = Table(show_header=False, box=None)
        servers_table.add_column("", style="cyan")
        servers_table.add_column("", style="white")
        
        servers = [
            ("fastapi", "خادم FastAPI مع Uvicorn"),
            ("flask", "خادم Flask"),
            ("django", "خادم Django"),
            ("tornado", "خادم Tornado")
        ]
        
        for server, desc in servers:
            servers_table.add_row(f"• {server}", desc)
        
        self.console.print(servers_table)
    
    def show_status(self):
        processes = self.process_manager.list_processes()
        
        if not processes:
            self.console.print("[yellow]لا توجد عمليات نشطة[/yellow]")
            return
        
        status_table = Table(title="حالة العمليات", show_header=True, header_style="bold green")
        status_table.add_column("ID", style="cyan")
        status_table.add_column("الحالة", style="white")
        status_table.add_column("PID", style="yellow")
        status_table.add_column("وقت التشغيل", style="blue")
        status_table.add_column("استخدام المعالج", style="red")
        status_table.add_column("الذاكرة", style="magenta")
        
        for process in processes:
            status_color = "green" if process['status'] == 'running' else "red" if process['status'] == 'failed' else "yellow"
            running_time = f"{process.get('running_time', 0):.1f}s" if process.get('running_time') else "N/A"
            cpu_percent = f"{process.get('cpu_percent', 0):.1f}%" if process.get('cpu_percent') else "N/A"
            memory_mb = f"{process.get('memory_info', {}).get('rss', 0) / 1024 / 1024:.1f}MB" if process.get('memory_info') else "N/A"
            
            status_table.add_row(
                process['id'],
                f"[{status_color}]{process['status']}[/{status_color}]",
                str(process.get('pid', 'N/A')),
                running_time,
                cpu_percent,
                memory_mb
            )
        
        self.console.print(Panel(status_table, border_style="blue"))
    
    def list_processes(self):
        self.show_status()
        
        system_stats = self.process_manager.get_system_stats()
        
        system_table = Table(title="إحصائيات النظام", show_header=True, header_style="bold blue")
        system_table.add_column("المعيار", style="cyan")
        system_table.add_column("القيمة", style="white")
        
        system_table.add_row("استخدام المعالج", f"{system_stats.get('cpu_percent', 0):.1f}%")
        system_table.add_row("استخدام الذاكرة", f"{system_stats.get('memory', {}).get('percent', 0):.1f}%")
        system_table.add_row("استخدام القرص", f"{system_stats.get('disk', {}).get('percent', 0):.1f}%")
        system_table.add_row("العمليات النشطة", str(system_stats.get('active_processes', 0)))
        system_table.add_row("إجمالي العمليات", str(system_stats.get('total_processes', 0)))
        
        self.console.print(Panel(system_table, border_style="green"))
    
    def start_server(self, server_type: str = None, *args):
        if not server_type:
            server_type = Prompt.ask(
                "اختر نوع الخادم",
                choices=["fastapi", "flask", "django", "tornado"],
                default="fastapi"
            )
        
        server_type = server_type.lower()
        
        try:
            if server_type == "fastapi":
                server = FastAPIServer(debug=True)
                process_id = f"fastapi_{int(time.time())}"
            elif server_type == "flask":
                server = FlaskServer(debug=True)
                process_id = f"flask_{int(time.time())}"
            elif server_type == "django":
                server = DjangoServer(debug=True)
                process_id = f"django_{int(time.time())}"
            elif server_type == "tornado":
                server = TornadoServer(debug=True)
                process_id = f"tornado_{int(time.time())}"
            else:
                self.console.print(f"[red]نوع خادم غير مدعوم: {server_type}[/red]")
                return
            
            process = self.process_manager.create_process(
                process_id=process_id,
                target=server.run_in_background,
                persistent=True
            )
            
            if self.process_manager.start_process(process_id):
                self.console.print(f"[green]✅ تم تشغيل خادم {server_type} بنجاح[/green]")
                self.console.print(f"[blue]معرف العملية: {process_id}[/blue]")
                
                if hasattr(server, 'port'):
                    self.console.print(f"[yellow]الخادم يعمل على: http://0.0.0.0:{server.port}[/yellow]")
            else:
                self.console.print(f"[red]❌ فشل في تشغيل خادم {server_type}[/red]")
        
        except Exception as e:
            self.console.print(f"[red]خطأ في تشغيل الخادم: {e}[/red]")
    
    def stop_process(self, process_id: str = None):
        if not process_id:
            processes = self.process_manager.list_processes()
            if not processes:
                self.console.print("[yellow]لا توجد عمليات للإيقاف[/yellow]")
                return
            
            choices = [p['id'] for p in processes if p['status'] == 'running']
            if not choices:
                self.console.print("[yellow]لا توجد عمليات نشطة للإيقاف[/yellow]")
                return
            
            process_id = Prompt.ask(
                "اختر العملية للإيقاف",
                choices=choices
            )
        
        if self.process_manager.stop_process(process_id):
            self.console.print(f"[green]✅ تم إيقاف العملية {process_id}[/green]")
        else:
            self.console.print(f"[red]❌ فشل في إيقاف العملية {process_id}[/red]")
    
    def restart_process(self, process_id: str = None):
        if not process_id:
            processes = self.process_manager.list_processes()
            if not processes:
                self.console.print("[yellow]لا توجد عمليات لإعادة التشغيل[/yellow]")
                return
            
            choices = [p['id'] for p in processes]
            process_id = Prompt.ask(
                "اختر العملية لإعادة التشغيل",
                choices=choices
            )
        
        if self.process_manager.restart_process(process_id):
            self.console.print(f"[green]✅ تم إعادة تشغيل العملية {process_id}[/green]")
        else:
            self.console.print(f"[red]❌ فشل في إعادة تشغيل العملية {process_id}[/red]")
    
    def show_logs(self, process_id: str = None, lines: int = 50):
        if not process_id:
            processes = self.process_manager.list_processes()
            if not processes:
                self.console.print("[yellow]لا توجد عمليات لعرض سجلاتها[/yellow]")
                return
            
            choices = [p['id'] for p in processes]
            process_id = Prompt.ask("اختر العملية لعرض سجلاتها", choices=choices)
        
        logs = self.persistence_manager.get_process_logs(process_id, lines)
        
        if not logs:
            self.console.print(f"[yellow]لا توجد سجلات للعملية {process_id}[/yellow]")
            return
        
        logs_table = Table(title=f"سجلات العملية: {process_id}", show_header=True, header_style="bold blue")
        logs_table.add_column("الوقت", style="cyan")
        logs_table.add_column("المستوى", style="white")
        logs_table.add_column("الرسالة", style="white")
        
        for log in logs:
            timestamp = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(log['timestamp']))
            level_color = "green" if log['level'] == 'INFO' else "yellow" if log['level'] == 'WARNING' else "red"
            
            logs_table.add_row(
                timestamp,
                f"[{level_color}]{log['level']}[/{level_color}]",
                log['message'][:80] + "..." if len(log['message']) > 80 else log['message']
            )
        
        self.console.print(Panel(logs_table, border_style="blue"))
    
    def show_system_info(self):
        system_info = self.termux_optimizer.get_system_info()
        
        info_table = Table(title="معلومات النظام", show_header=True, header_style="bold green")
        info_table.add_column("المعلومة", style="cyan")
        info_table.add_column("القيمة", style="white")
        
        info_table.add_row("نوع النظام", "Termux" if system_info['is_termux'] else "Linux")
        info_table.add_row("منصة العمل", system_info['platform'])
        info_table.add_row("نسخة Python", system_info['python_version'])
        
        if system_info['is_termux']:
            info_table.add_row("نسخة Termux", system_info.get('termux_version', 'غير معروف'))
            info_table.add_row("نسخة Android", system_info.get('android_version', 'غير معروف'))
            info_table.add_row("موديل الجهاز", system_info.get('device_model', 'غير معروف'))
        
        optimizations = system_info.get('optimizations_applied', {})
        if optimizations:
            for opt_type, opts in optimizations.items():
                info_table.add_row(f"تحسينات {opt_type}", ", ".join(opts) if opts else "لا توجد")
        
        self.console.print(Panel(info_table, border_style="green"))
        
        system_stats = self.process_manager.get_system_stats()
        
        stats_table = Table(title="إحصائيات الأداء", show_header=True, header_style="bold blue")
        stats_table.add_column("المعيار", style="cyan")
        stats_table.add_column("القيمة", style="white")
        stats_table.add_column("الحالة", style="green")
        
        cpu_status = "جيد" if system_stats.get('cpu_percent', 0) < 70 else "مرتفع"
        memory_status = "جيد" if system_stats.get('memory', {}).get('percent', 0) < 80 else "مرتفع"
        disk_status = "جيد" if system_stats.get('disk', {}).get('percent', 0) < 90 else "ممتلئ"
        
        stats_table.add_row("استخدام المعالج", f"{system_stats.get('cpu_percent', 0):.1f}%", cpu_status)
        stats_table.add_row("استخدام الذاكرة", f"{system_stats.get('memory', {}).get('percent', 0):.1f}%", memory_status)
        stats_table.add_row("استخدام القرص", f"{system_stats.get('disk', {}).get('percent', 0):.1f}%", disk_status)
        
        self.console.print(Panel(stats_table, border_style="blue"))
    
    def run_optimization(self):
        self.console.print("[blue]🚀 بدء تحسين النظام...[/blue]")
        
        with self.console.status("[bold green]تطبيق التحسينات...") as status:
            optimizations = []
            
            if self.termux_optimizer.optimize_for_android():
                optimizations.append("تحسينات Android")
            
            termux_opts = self.termux_optimizer.apply_termux_specific_optimizations()
            if termux_opts:
                optimizations.extend(termux_opts)
            
            prevention_strategies = self.termux_optimizer.prevent_android_killing()
            if prevention_strategies:
                optimizations.extend(prevention_strategies)
        
        if optimizations:
            self.console.print("[green]✅ تم تطبيق التحسينات التالية:[/green]")
            for opt in optimizations:
                self.console.print(f"  • {opt}")
        else:
            self.console.print("[yellow]⚠️ لم يتم تطبيق أي تحسينات[/yellow]")
    
    def create_backup(self, name: str = None):
        if not name:
            name = f"backup_{int(time.time())}"
        
        with self.console.status("[bold blue]إنشاء نسخة احتياطية..."):
            backup_path = self.persistence_manager.backup_data(name)
        
        if backup_path:
            self.console.print(f"[green]✅ تم إنشاء النسخة الاحتياطية: {backup_path}[/green]")
        else:
            self.console.print("[red]❌ فشل في إنشاء النسخة الاحتياطية[/red]")
    
    def restore_backup(self, name: str = None):
        if not name:
            self.console.print("[red]يجب تحديد اسم النسخة الاحتياطية[/red]")
            return
        
        if Confirm.ask(f"[yellow]هل أنت متأكد من استعادة النسخة الاحتياطية '{name}'؟[/yellow]"):
            with self.console.status("[bold blue]استعادة النسخة الاحتياطية..."):
                success = self.persistence_manager.restore_from_backup(name)
            
            if success:
                self.console.print("[green]✅ تم استعادة النسخة الاحتياطية بنجاح[/green]")
            else:
                self.console.print("[red]❌ فشل في استعادة النسخة الاحتياطية[/red]")
    
    def manage_config(self, key: str = None, value: str = None):
        if key and value:
            self.persistence_manager.save_configuration(key, value)
            self.console.print(f"[green]✅ تم حفظ التكوين: {key} = {value}[/green]")
        elif key:
            config_value = self.persistence_manager.load_configuration(key)
            self.console.print(f"[blue]{key}: {config_value}[/blue]")
        else:
            self.console.print("[blue]أوامر التكوين:[/blue]")
            self.console.print("  config <key> <value> - حفظ قيمة")
            self.console.print("  config <key> - عرض قيمة")
    
    def start_monitoring(self):
        self.console.print("[blue]🔍 بدء مراقبة الموارد...[/blue]")
        
        monitor = self.termux_optimizer.monitor_system_resources(interval=5)
        
        try:
            with Live(console=self.console, refresh_per_second=1) as live:
                for i in range(60):
                    system_stats = self.process_manager.get_system_stats()
                    
                    layout = Layout()
                    
                    stats_table = Table(title="مراقبة الموارد المباشرة", show_header=True, header_style="bold green")
                    stats_table.add_column("المورد", style="cyan")
                    stats_table.add_column("القيمة", style="white")
                    stats_table.add_column("الحالة", style="green")
                    
                    cpu = system_stats.get('cpu_percent', 0)
                    memory = system_stats.get('memory', {}).get('percent', 0)
                    
                    cpu_status = "🟢 جيد" if cpu < 70 else "🟡 متوسط" if cpu < 90 else "🔴 مرتفع"
                    memory_status = "🟢 جيد" if memory < 70 else "🟡 متوسط" if memory < 90 else "🔴 مرتفع"
                    
                    stats_table.add_row("المعالج", f"{cpu:.1f}%", cpu_status)
                    stats_table.add_row("الذاكرة", f"{memory:.1f}%", memory_status)
                    stats_table.add_row("العمليات النشطة", str(system_stats.get('active_processes', 0)), "🔵 نشط")
                    
                    layout.update(stats_table)
                    live.update(layout)
                    
                    time.sleep(1)
        
        except KeyboardInterrupt:
            self.console.print("\n[yellow]تم إيقاف المراقبة[/yellow]")
    
    def open_dashboard(self):
        self.console.print("[blue]🌐 فتح لوحة التحكم...[/blue]")
        
        servers = [
            ("FastAPI", "http://localhost:8000/docs"),
            ("Flask", "http://localhost:5000/dashboard"),
            ("Django", "http://localhost:8000/dashboard"),
            ("Tornado", "http://localhost:8888/dashboard")
        ]
        
        dashboard_table = Table(title="لوحات التحكم المتاحة", show_header=True, header_style="bold blue")
        dashboard_table.add_column("الخادم", style="cyan")
        dashboard_table.add_column("الرابط", style="yellow")
        
        for server, url in servers:
            dashboard_table.add_row(server, url)
        
        self.console.print(Panel(dashboard_table, border_style="green"))
        
        if Confirm.ask("هل تريد فتح لوحة تحكم في المتصفح؟"):
            url = Prompt.ask("أدخل الرابط", default="http://localhost:8000")
            try:
                subprocess.run(['termux-open-url', url], check=True)
                self.console.print(f"[green]✅ تم فتح {url}[/green]")
            except subprocess.CalledProcessError:
                self.console.print(f"[yellow]لم يتمكن من فتح الرابط تلقائياً: {url}[/yellow]")
    
    def exit_interactive(self):
        self.console.print("[yellow]👋 شكراً لاستخدام RunMero![/yellow]")
        self.interactive_mode = False
    
    def cleanup_and_exit(self):
        self.console.print("[blue]🧹 تنظيف الموارد...[/blue]")
        
        try:
            self.process_manager.stop_all_processes()
            self.persistence_manager.force_save_all()
        except Exception as e:
            self.console.print(f"[yellow]تحذير أثناء التنظيف: {e}[/yellow]")
        
        self.console.print("[green]✅ تم الخروج من RunMero بأمان[/green]")
        sys.exit(0)

@click.group()
@click.version_option("2.5.0", prog_name="RunMero")
def cli():
    pass

@cli.command()
def interactive():
    cli_instance = RunMeroCLI()
    cli_instance.start_interactive_mode()

@cli.command()
@click.argument('server_type', type=click.Choice(['fastapi', 'flask', 'django', 'tornado']))
@click.option('--port', default=None, type=int, help='منفذ الخادم')
@click.option('--host', default='0.0.0.0', help='عنوان الخادم')
def start(server_type, port, host):
    cli_instance = RunMeroCLI()
    cli_instance.start_server(server_type)

@cli.command()
def status():
    cli_instance = RunMeroCLI()
    cli_instance.show_status()

@cli.command()
def optimize():
    cli_instance = RunMeroCLI()
    cli_instance.run_optimization()

def main():
    try:
        cli_instance = RunMeroCLI()
        cli_instance.start_interactive_mode()
    except KeyboardInterrupt:
        print("\nتم الخروج من RunMero")
        sys.exit(0)
    except Exception as e:
        print(f"خطأ: {e}")
        sys.exit(1)

if __name__ == "__main__":
    main()



# حقوق الطبع والنشر محفوظة © 2025 mero - من أرض الكرم حبرون فلسطين

import os
import json
import time
import threading
import subprocess
import signal
import psutil
from typing import Dict, List, Any, Optional
import sqlite3
from pathlib import Path

class PersistenceManager:
    def __init__(self):
        self.data_dir = self._get_data_directory()
        self.db_path = os.path.join(self.data_dir, 'runmero_persistence.db')
        self.processes_file = os.path.join(self.data_dir, 'processes.json')
        self.config_file = os.path.join(self.data_dir, 'config.json')
        self.lock_file = os.path.join(self.data_dir, 'runmero.lock')
        
        self._ensure_data_directory()
        self._init_database()
        self._setup_signal_handlers()
        
        self.auto_save_thread = None
        self.auto_save_enabled = True
        self._start_auto_save()
        
    def _get_data_directory(self) -> str:
        if os.getenv('TERMUX_VERSION'):
            base_dir = '/data/data/com.termux/files/usr/var/lib/runmero'
        else:
            home_dir = os.path.expanduser('~')
            base_dir = os.path.join(home_dir, '.runmero')
        return base_dir
    
    def _ensure_data_directory(self):
        Path(self.data_dir).mkdir(parents=True, exist_ok=True)
        
        subdirs = ['processes', 'logs', 'configs', 'backups', 'temp']
        for subdir in subdirs:
            Path(os.path.join(self.data_dir, subdir)).mkdir(exist_ok=True)
    
    def _init_database(self):
        try:
            with sqlite3.connect(self.db_path) as conn:
                cursor = conn.cursor()
                
                cursor.execute('''
                    CREATE TABLE IF NOT EXISTS processes (
                        id TEXT PRIMARY KEY,
                        pid INTEGER,
                        status TEXT,
                        start_time REAL,
                        end_time REAL,
                        command TEXT,
                        args TEXT,
                        kwargs TEXT,
                        metadata TEXT,
                        created_at REAL,
                        updated_at REAL
                    )
                ''')
                
                cursor.execute('''
                    CREATE TABLE IF NOT EXISTS process_logs (
                        id INTEGER PRIMARY KEY AUTOINCREMENT,
                        process_id TEXT,
                        timestamp REAL,
                        level TEXT,
                        message TEXT,
                        FOREIGN KEY (process_id) REFERENCES processes(id)
                    )
                ''')
                
                cursor.execute('''
                    CREATE TABLE IF NOT EXISTS system_events (
                        id INTEGER PRIMARY KEY AUTOINCREMENT,
                        timestamp REAL,
                        event_type TEXT,
                        event_data TEXT,
                        severity TEXT
                    )
                ''')
                
                cursor.execute('''
                    CREATE TABLE IF NOT EXISTS configurations (
                        key TEXT PRIMARY KEY,
                        value TEXT,
                        created_at REAL,
                        updated_at REAL
                    )
                ''')
                
                conn.commit()
        except Exception as e:
            print(f"خطأ في تهيئة قاعدة البيانات: {e}")
    
    def _setup_signal_handlers(self):
        def save_on_signal(signum, frame):
            self.force_save_all()
        
        signal.signal(signal.SIGTERM, save_on_signal)
        signal.signal(signal.SIGINT, save_on_signal)
        
        if hasattr(signal, 'SIGUSR1'):
            signal.signal(signal.SIGUSR1, save_on_signal)
    
    def save_process(self, process_id: str, process_data: Dict[str, Any]):
        try:
            with sqlite3.connect(self.db_path) as conn:
                cursor = conn.cursor()
                
                now = time.time()
                
                cursor.execute('''
                    INSERT OR REPLACE INTO processes 
                    (id, pid, status, start_time, end_time, command, args, kwargs, metadata, created_at, updated_at)
                    VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
                ''', (
                    process_id,
                    process_data.get('pid'),
                    process_data.get('status', 'created'),
                    process_data.get('start_time'),
                    process_data.get('end_time'),
                    process_data.get('command'),
                    json.dumps(process_data.get('args', [])),
                    json.dumps(process_data.get('kwargs', {})),
                    json.dumps(process_data.get('metadata', {})),
                    process_data.get('created_at', now),
                    now
                ))
                
                conn.commit()
                
        except Exception as e:
            print(f"خطأ في حفظ العملية {process_id}: {e}")
    
    def load_processes(self) -> Dict[str, Dict[str, Any]]:
        processes = {}
        try:
            with sqlite3.connect(self.db_path) as conn:
                cursor = conn.cursor()
                
                cursor.execute('SELECT * FROM processes')
                rows = cursor.fetchall()
                
                columns = [desc[0] for desc in cursor.description]
                
                for row in rows:
                    process_data = dict(zip(columns, row))
                    
                    try:
                        process_data['args'] = json.loads(process_data['args'] or '[]')
                        process_data['kwargs'] = json.loads(process_data['kwargs'] or '{}')
                        process_data['metadata'] = json.loads(process_data['metadata'] or '{}')
                    except json.JSONDecodeError:
                        process_data['args'] = []
                        process_data['kwargs'] = {}
                        process_data['metadata'] = {}
                    
                    processes[process_data['id']] = process_data
                    
        except Exception as e:
            print(f"خطأ في تحميل العمليات: {e}")
        
        return processes
    
    def update_process(self, process_id: str, updates: Dict[str, Any]):
        try:
            with sqlite3.connect(self.db_path) as conn:
                cursor = conn.cursor()
                
                set_clauses = []
                values = []
                
                for key, value in updates.items():
                    if key in ['args', 'kwargs', 'metadata']:
                        value = json.dumps(value)
                    set_clauses.append(f"{key} = ?")
                    values.append(value)
                
                set_clauses.append("updated_at = ?")
                values.append(time.time())
                values.append(process_id)
                
                query = f"UPDATE processes SET {', '.join(set_clauses)} WHERE id = ?"
                cursor.execute(query, values)
                
                conn.commit()
                
        except Exception as e:
            print(f"خطأ في تحديث العملية {process_id}: {e}")
    
    def remove_process(self, process_id: str):
        try:
            with sqlite3.connect(self.db_path) as conn:
                cursor = conn.cursor()
                
                cursor.execute('DELETE FROM process_logs WHERE process_id = ?', (process_id,))
                cursor.execute('DELETE FROM processes WHERE id = ?', (process_id,))
                
                conn.commit()
                
        except Exception as e:
            print(f"خطأ في حذف العملية {process_id}: {e}")
    
    def log_process_event(self, process_id: str, level: str, message: str):
        try:
            with sqlite3.connect(self.db_path) as conn:
                cursor = conn.cursor()
                
                cursor.execute('''
                    INSERT INTO process_logs (process_id, timestamp, level, message)
                    VALUES (?, ?, ?, ?)
                ''', (process_id, time.time(), level, message))
                
                conn.commit()
                
        except Exception as e:
            print(f"خطأ في تسجيل حدث العملية: {e}")
    
    def get_process_logs(self, process_id: str, limit: int = 100) -> List[Dict[str, Any]]:
        logs = []
        try:
            with sqlite3.connect(self.db_path) as conn:
                cursor = conn.cursor()
                
                cursor.execute('''
                    SELECT timestamp, level, message FROM process_logs 
                    WHERE process_id = ? 
                    ORDER BY timestamp DESC 
                    LIMIT ?
                ''', (process_id, limit))
                
                for row in cursor.fetchall():
                    logs.append({
                        'timestamp': row[0],
                        'level': row[1],
                        'message': row[2]
                    })
                    
        except Exception as e:
            print(f"خطأ في جلب سجلات العملية: {e}")
        
        return logs
    
    def log_system_event(self, event_type: str, event_data: Any, severity: str = 'info'):
        try:
            with sqlite3.connect(self.db_path) as conn:
                cursor = conn.cursor()
                
                cursor.execute('''
                    INSERT INTO system_events (timestamp, event_type, event_data, severity)
                    VALUES (?, ?, ?, ?)
                ''', (time.time(), event_type, json.dumps(event_data), severity))
                
                conn.commit()
                
        except Exception as e:
            print(f"خطأ في تسجيل حدث النظام: {e}")
    
    def get_system_events(self, event_type: Optional[str] = None, limit: int = 100) -> List[Dict[str, Any]]:
        events = []
        try:
            with sqlite3.connect(self.db_path) as conn:
                cursor = conn.cursor()
                
                if event_type:
                    query = '''
                        SELECT timestamp, event_type, event_data, severity FROM system_events 
                        WHERE event_type = ? 
                        ORDER BY timestamp DESC 
                        LIMIT ?
                    '''
                    cursor.execute(query, (event_type, limit))
                else:
                    query = '''
                        SELECT timestamp, event_type, event_data, severity FROM system_events 
                        ORDER BY timestamp DESC 
                        LIMIT ?
                    '''
                    cursor.execute(query, (limit,))
                
                for row in cursor.fetchall():
                    try:
                        event_data = json.loads(row[2])
                    except json.JSONDecodeError:
                        event_data = row[2]
                    
                    events.append({
                        'timestamp': row[0],
                        'event_type': row[1],
                        'event_data': event_data,
                        'severity': row[3]
                    })
                    
        except Exception as e:
            print(f"خطأ في جلب أحداث النظام: {e}")
        
        return events
    
    def save_configuration(self, key: str, value: Any):
        try:
            with sqlite3.connect(self.db_path) as conn:
                cursor = conn.cursor()
                
                now = time.time()
                cursor.execute('''
                    INSERT OR REPLACE INTO configurations (key, value, created_at, updated_at)
                    VALUES (?, ?, COALESCE((SELECT created_at FROM configurations WHERE key = ?), ?), ?)
                ''', (key, json.dumps(value), key, now, now))
                
                conn.commit()
                
        except Exception as e:
            print(f"خطأ في حفظ التكوين {key}: {e}")
    
    def load_configuration(self, key: str, default: Any = None) -> Any:
        try:
            with sqlite3.connect(self.db_path) as conn:
                cursor = conn.cursor()
                
                cursor.execute('SELECT value FROM configurations WHERE key = ?', (key,))
                row = cursor.fetchone()
                
                if row:
                    return json.loads(row[0])
                    
        except Exception as e:
            print(f"خطأ في تحميل التكوين {key}: {e}")
        
        return default
    
    def backup_data(self, backup_name: Optional[str] = None) -> str:
        if not backup_name:
            backup_name = f"backup_{int(time.time())}"
        
        backup_dir = os.path.join(self.data_dir, 'backups', backup_name)
        Path(backup_dir).mkdir(parents=True, exist_ok=True)
        
        try:
            backup_db = os.path.join(backup_dir, 'runmero_persistence.db')
            
            with sqlite3.connect(self.db_path) as source:
                with sqlite3.connect(backup_db) as backup:
                    source.backup(backup)
            
            backup_info = {
                'backup_name': backup_name,
                'timestamp': time.time(),
                'database_size': os.path.getsize(backup_db) if os.path.exists(backup_db) else 0,
                'version': '2.5.0'
            }
            
            info_file = os.path.join(backup_dir, 'backup_info.json')
            with open(info_file, 'w') as f:
                json.dump(backup_info, f, indent=2)
            
            return backup_dir
            
        except Exception as e:
            print(f"خطأ في إنشاء النسخة الاحتياطية: {e}")
            return ""
    
    def restore_from_backup(self, backup_name: str) -> bool:
        backup_dir = os.path.join(self.data_dir, 'backups', backup_name)
        backup_db = os.path.join(backup_dir, 'runmero_persistence.db')
        
        if not os.path.exists(backup_db):
            return False
        
        try:
            current_db_backup = f"{self.db_path}.before_restore"
            if os.path.exists(self.db_path):
                os.rename(self.db_path, current_db_backup)
            
            with sqlite3.connect(backup_db) as source:
                with sqlite3.connect(self.db_path) as target:
                    source.backup(target)
            
            return True
            
        except Exception as e:
            print(f"خطأ في استعادة النسخة الاحتياطية: {e}")
            if os.path.exists(f"{self.db_path}.before_restore"):
                os.rename(f"{self.db_path}.before_restore", self.db_path)
            return False
    
    def cleanup_old_data(self, days: int = 30):
        cutoff_time = time.time() - (days * 24 * 3600)
        
        try:
            with sqlite3.connect(self.db_path) as conn:
                cursor = conn.cursor()
                
                cursor.execute('DELETE FROM process_logs WHERE timestamp < ?', (cutoff_time,))
                cursor.execute('DELETE FROM system_events WHERE timestamp < ?', (cutoff_time,))
                
                cursor.execute('''
                    DELETE FROM processes 
                    WHERE status IN ('stopped', 'failed', 'completed') 
                    AND updated_at < ?
                ''', (cutoff_time,))
                
                conn.commit()
                
                cursor.execute('VACUUM')
                
        except Exception as e:
            print(f"خطأ في تنظيف البيانات القديمة: {e}")
    
    def _start_auto_save(self):
        self.auto_save_thread = threading.Thread(target=self._auto_save_loop, daemon=True)
        self.auto_save_thread.start()
    
    def _auto_save_loop(self):
        while self.auto_save_enabled:
            try:
                time.sleep(300)
                self._perform_maintenance()
            except Exception as e:
                print(f"خطأ في الحفظ التلقائي: {e}")
    
    def _perform_maintenance(self):
        try:
            with sqlite3.connect(self.db_path) as conn:
                cursor = conn.cursor()
                
                cursor.execute('DELETE FROM process_logs WHERE id NOT IN (SELECT id FROM process_logs ORDER BY timestamp DESC LIMIT 10000)')
                cursor.execute('DELETE FROM system_events WHERE id NOT IN (SELECT id FROM system_events ORDER BY timestamp DESC LIMIT 5000)')
                
                conn.commit()
                
                if time.time() % 86400 < 300:
                    cursor.execute('VACUUM')
                    
        except Exception as e:
            print(f"خطأ في صيانة قاعدة البيانات: {e}")
    
    def force_save_all(self):
        try:
            processes = {}
            for pid in psutil.pids():
                try:
                    process = psutil.Process(pid)
                    if 'runmero' in ' '.join(process.cmdline()).lower():
                        processes[str(pid)] = {
                            'pid': pid,
                            'status': process.status(),
                            'name': process.name(),
                            'cmdline': process.cmdline(),
                            'create_time': process.create_time(),
                            'saved_at': time.time()
                        }
                except (psutil.NoSuchProcess, psutil.AccessDenied):
                    pass
            
            emergency_file = os.path.join(self.data_dir, 'emergency_processes.json')
            with open(emergency_file, 'w') as f:
                json.dump(processes, f, indent=2)
                
        except Exception as e:
            print(f"خطأ في الحفظ الطارئ: {e}")
    
    def get_persistence_stats(self) -> Dict[str, Any]:
        stats = {
            'data_directory': self.data_dir,
            'database_size': 0,
            'total_processes': 0,
            'active_processes': 0,
            'total_logs': 0,
            'total_events': 0,
            'configurations_count': 0,
            'disk_usage': 0
        }
        
        try:
            if os.path.exists(self.db_path):
                stats['database_size'] = os.path.getsize(self.db_path)
            
            with sqlite3.connect(self.db_path) as conn:
                cursor = conn.cursor()
                
                cursor.execute('SELECT COUNT(*) FROM processes')
                stats['total_processes'] = cursor.fetchone()[0]
                
                cursor.execute('SELECT COUNT(*) FROM processes WHERE status = ?', ('running',))
                stats['active_processes'] = cursor.fetchone()[0]
                
                cursor.execute('SELECT COUNT(*) FROM process_logs')
                stats['total_logs'] = cursor.fetchone()[0]
                
                cursor.execute('SELECT COUNT(*) FROM system_events')
                stats['total_events'] = cursor.fetchone()[0]
                
                cursor.execute('SELECT COUNT(*) FROM configurations')
                stats['configurations_count'] = cursor.fetchone()[0]
            
            total_size = 0
            for root, dirs, files in os.walk(self.data_dir):
                for file in files:
                    file_path = os.path.join(root, file)
                    total_size += os.path.getsize(file_path)
            stats['disk_usage'] = total_size
            
        except Exception as e:
            print(f"خطأ في جلب إحصائيات المثابرة: {e}")
        
        return stats
    
    def __del__(self):
        self.auto_save_enabled = False
        if self.auto_save_thread and self.auto_save_thread.is_alive():
            self.auto_save_thread.join(timeout=5)



# حقوق الطبع والنشر محفوظة © 2025 mero - من أرض البرتقال يافا فلسطين

import asyncio
import uvicorn
from fastapi import FastAPI, BackgroundTasks, HTTPException, Depends
from fastapi.middleware.cors import CORSMiddleware
from fastapi.middleware.gzip import GZipMiddleware
from fastapi.responses import JSONResponse
from pydantic import BaseModel
from typing import Optional, Dict, Any, List
import threading
import time
import os

class FastAPIServer:
    def __init__(self, 
                 app_name: str = "RunMero FastAPI Server",
                 host: str = "0.0.0.0", 
                 port: int = 8000,
                 debug: bool = False,
                 auto_reload: bool = False):
        
        self.app_name = app_name
        self.host = host
        self.port = port
        self.debug = debug
        self.auto_reload = auto_reload
        
        self.app = FastAPI(
            title=app_name,
            description="خادم FastAPI قوي مدعوم بمكتبة RunMero - صنع في فلسطين",
            version="2.5.0",
            docs_url="/docs" if debug else None,
            redoc_url="/redoc" if debug else None
        )
        
        self._setup_middleware()
        self._setup_routes()
        self._background_tasks = []
        self._running = False
        self._server_thread = None
        
    def _setup_middleware(self):
        self.app.add_middleware(
            CORSMiddleware,
            allow_origins=["*"],
            allow_credentials=True,
            allow_methods=["*"],
            allow_headers=["*"],
        )
        
        self.app.add_middleware(GZipMiddleware, minimum_size=1000)
        
        @self.app.middleware("http")
        async def process_time_header(request, call_next):
            start_time = time.time()
            response = await call_next(request)
            process_time = time.time() - start_time
            response.headers["X-Process-Time"] = str(process_time)
            response.headers["X-Powered-By"] = "RunMero FastAPI - فلسطين"
            return response

    def _setup_routes(self):
        @self.app.get("/")
        async def root():
            return {
                "message": "مرحباً بك في خادم FastAPI المدعوم بـ RunMero",
                "version": "2.5.0",
                "author": "mero",
                "country": "فلسطين الحبيبة",
                "status": "running",
                "uptime": time.time() - getattr(self, '_start_time', time.time())
            }

        @self.app.get("/health")
        async def health_check():
            return {
                "status": "healthy",
                "timestamp": time.time(),
                "server": self.app_name,
                "process_id": os.getpid()
            }

        @self.app.get("/status")
        async def server_status():
            import psutil
            process = psutil.Process()
            
            return {
                "server_info": {
                    "name": self.app_name,
                    "host": self.host,
                    "port": self.port,
                    "running": self._running
                },
                "system_info": {
                    "cpu_percent": process.cpu_percent(),
                    "memory_info": process.memory_info()._asdict(),
                    "connections": len(process.connections()) if hasattr(process, 'connections') else 0,
                    "threads": process.num_threads()
                },
                "background_tasks": len(self._background_tasks)
            }

        @self.app.post("/background-task")
        async def create_background_task(background_tasks: BackgroundTasks):
            task_id = f"task_{int(time.time())}"
            
            def long_running_task(task_id: str):
                time.sleep(10)
                print(f"انتهت المهمة في الخلفية: {task_id}")
            
            background_tasks.add_task(long_running_task, task_id)
            self._background_tasks.append(task_id)
            
            return {"message": f"تم إنشاء مهمة في الخلفية: {task_id}"}

        @self.app.websocket("/ws")
        async def websocket_endpoint(websocket):
            await websocket.accept()
            try:
                while True:
                    data = await websocket.receive_text()
                    await websocket.send_text(f"تم استلام الرسالة: {data}")
            except Exception as e:
                print(f"خطأ في WebSocket: {e}")

    def add_route(self, path: str, endpoint, methods: List[str] = ["GET"]):
        for method in methods:
            if method.upper() == "GET":
                self.app.get(path)(endpoint)
            elif method.upper() == "POST":
                self.app.post(path)(endpoint)
            elif method.upper() == "PUT":
                self.app.put(path)(endpoint)
            elif method.upper() == "DELETE":
                self.app.delete(path)(endpoint)
            elif method.upper() == "PATCH":
                self.app.patch(path)(endpoint)

    def add_middleware(self, middleware_class, **kwargs):
        self.app.add_middleware(middleware_class, **kwargs)

    def include_router(self, router, prefix: str = "", tags: List[str] = None):
        self.app.include_router(router, prefix=prefix, tags=tags or [])

    def run_in_background(self):
        if self._running:
            return False
        
        def server_worker():
            self._running = True
            self._start_time = time.time()
            
            config = uvicorn.Config(
                app=self.app,
                host=self.host,
                port=self.port,
                reload=self.auto_reload,
                log_level="info" if self.debug else "warning",
                access_log=self.debug
            )
            
            server = uvicorn.Server(config)
            
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)
            
            try:
                loop.run_until_complete(server.serve())
            except Exception as e:
                print(f"خطأ في تشغيل الخادم: {e}")
            finally:
                self._running = False
                loop.close()
        
        self._server_thread = threading.Thread(target=server_worker, daemon=True)
        self._server_thread.start()
        
        time.sleep(2)
        return self._running

    def run_sync(self):
        self._running = True
        self._start_time = time.time()
        
        try:
            uvicorn.run(
                app=self.app,
                host=self.host,
                port=self.port,
                reload=self.auto_reload,
                log_level="info" if self.debug else "warning",
                access_log=self.debug
            )
        except KeyboardInterrupt:
            print("تم إيقاف الخادم بواسطة المستخدم")
        except Exception as e:
            print(f"خطأ في تشغيل الخادم: {e}")
        finally:
            self._running = False

    def stop(self):
        self._running = False
        if self._server_thread and self._server_thread.is_alive():
            self._server_thread.join(timeout=10)

    def is_running(self) -> bool:
        return self._running

    def get_app(self):
        return self.app

    def add_startup_event(self, func):
        self.app.add_event_handler("startup", func)

    def add_shutdown_event(self, func):
        self.app.add_event_handler("shutdown", func)

    def create_advanced_endpoints(self):
        class ProcessModel(BaseModel):
            name: str
            command: str
            args: Optional[List[str]] = []
            env: Optional[Dict[str, str]] = {}

        @self.app.post("/processes")
        async def create_process(process: ProcessModel):
            return {
                "message": f"تم إنشاء العملية: {process.name}",
                "process": process.dict(),
                "timestamp": time.time()
            }

        @self.app.get("/processes")
        async def list_processes():
            import psutil
            processes = []
            for proc in psutil.process_iter(['pid', 'name', 'cpu_percent', 'memory_percent']):
                try:
                    processes.append(proc.info)
                except (psutil.NoSuchProcess, psutil.AccessDenied):
                    pass
            
            return {
                "processes": processes[:20],
                "total": len(processes),
                "timestamp": time.time()
            }

        @self.app.get("/system")
        async def system_info():
            import psutil
            return {
                "cpu": {
                    "percent": psutil.cpu_percent(interval=1),
                    "count": psutil.cpu_count(),
                    "freq": psutil.cpu_freq()._asdict() if psutil.cpu_freq() else None
                },
                "memory": psutil.virtual_memory()._asdict(),
                "disk": psutil.disk_usage('/')._asdict(),
                "network": {name: stats._asdict() for name, stats in psutil.net_io_counters(pernic=True).items()},
                "uptime": time.time() - psutil.boot_time()
            }

    def setup_file_server(self, static_path: str = "static"):
        from fastapi.staticfiles import StaticFiles
        
        if os.path.exists(static_path):
            self.app.mount("/static", StaticFiles(directory=static_path), name="static")

    def setup_websocket_manager(self):
        from fastapi import WebSocket, WebSocketDisconnect
        
        class ConnectionManager:
            def __init__(self):
                self.active_connections: List[WebSocket] = []

            async def connect(self, websocket: WebSocket):
                await websocket.accept()
                self.active_connections.append(websocket)

            def disconnect(self, websocket: WebSocket):
                self.active_connections.remove(websocket)

            async def send_personal_message(self, message: str, websocket: WebSocket):
                await websocket.send_text(message)

            async def broadcast(self, message: str):
                for connection in self.active_connections:
                    try:
                        await connection.send_text(message)
                    except:
                        self.disconnect(connection)

        manager = ConnectionManager()

        @self.app.websocket("/ws/{client_id}")
        async def websocket_endpoint(websocket: WebSocket, client_id: int):
            await manager.connect(websocket)
            try:
                while True:
                    data = await websocket.receive_text()
                    await manager.send_personal_message(f"رسالة من العميل #{client_id}: {data}", websocket)
                    await manager.broadcast(f"العميل #{client_id} قال: {data}")
            except WebSocketDisconnect:
                manager.disconnect(websocket)
                await manager.broadcast(f"العميل #{client_id} قطع الاتصال")

        return manager



# حقوق الطبع والنشر محفوظة © 2025 mero - من قلب فلسطين الحبيبة

import os
import sys
import json
import time
import signal
import threading
import multiprocessing
from typing import Dict, List, Optional, Any, Callable
import psutil
from .process import BackgroundProcess
from .signal_handler import SignalHandler
from ..termux.persistence import PersistenceManager
from ..utils.console import Console

class ProcessManager:
    def __init__(self):
        self.processes: Dict[str, BackgroundProcess] = {}
        self.active_processes: Dict[str, multiprocessing.Process] = {}
        self.signal_handler = SignalHandler()
        self.persistence = PersistenceManager()
        self.console = Console()
        self._lock = threading.Lock()
        self._monitoring = False
        self._monitor_thread = None
        self._setup_signal_handlers()
        self._load_persistent_processes()

    def _setup_signal_handlers(self):
        for sig in [signal.SIGTERM, signal.SIGINT, signal.SIGQUIT]:
            signal.signal(sig, self._handle_shutdown)

    def _handle_shutdown(self, signum, frame):
        self.console.info(f"تم استلام إشارة الإيقاف: {signum}")
        self.stop_all_processes()
        sys.exit(0)

    def _load_persistent_processes(self):
        persistent_data = self.persistence.load_processes()
        for process_id, process_data in persistent_data.items():
            try:
                if psutil.pid_exists(process_data['pid']):
                    process = BackgroundProcess(
                        process_id=process_id,
                        target=None,
                        args=process_data.get('args', []),
                        kwargs=process_data.get('kwargs', {})
                    )
                    process._pid = process_data['pid']
                    process._status = 'running'
                    self.processes[process_id] = process
                    self.console.success(f"تم استعادة العملية: {process_id}")
                else:
                    self.persistence.remove_process(process_id)
            except Exception as e:
                self.console.error(f"خطأ في استعادة العملية {process_id}: {e}")

    def create_process(self, 
                      process_id: str, 
                      target: Callable,
                      args: tuple = (),
                      kwargs: dict = None,
                      daemon: bool = True,
                      persistent: bool = True) -> BackgroundProcess:
        
        if kwargs is None:
            kwargs = {}
            
        with self._lock:
            if process_id in self.processes:
                raise ValueError(f"العملية {process_id} موجودة بالفعل")
            
            process = BackgroundProcess(
                process_id=process_id,
                target=target,
                args=args,
                kwargs=kwargs,
                daemon=daemon
            )
            
            self.processes[process_id] = process
            
            if persistent:
                self.persistence.save_process(process_id, {
                    'target': target.__name__ if target else None,
                    'args': args,
                    'kwargs': kwargs,
                    'daemon': daemon,
                    'created_at': time.time()
                })
            
            self.console.success(f"تم إنشاء العملية: {process_id}")
            return process

    def start_process(self, process_id: str) -> bool:
        with self._lock:
            if process_id not in self.processes:
                self.console.error(f"العملية {process_id} غير موجودة")
                return False
            
            process = self.processes[process_id]
            
            try:
                mp_process = multiprocessing.Process(
                    target=process.target,
                    args=process.args,
                    kwargs=process.kwargs,
                    daemon=process.daemon
                )
                mp_process.start()
                
                self.active_processes[process_id] = mp_process
                process._pid = mp_process.pid
                process._status = 'running'
                process._start_time = time.time()
                
                self.persistence.update_process(process_id, {
                    'pid': mp_process.pid,
                    'status': 'running',
                    'start_time': time.time()
                })
                
                self.console.success(f"تم تشغيل العملية: {process_id} (PID: {mp_process.pid})")
                
                if not self._monitoring:
                    self._start_monitoring()
                
                return True
                
            except Exception as e:
                self.console.error(f"خطأ في تشغيل العملية {process_id}: {e}")
                process._status = 'failed'
                return False

    def stop_process(self, process_id: str, timeout: int = 10) -> bool:
        with self._lock:
            if process_id not in self.processes:
                self.console.error(f"العملية {process_id} غير موجودة")
                return False
            
            process = self.processes[process_id]
            
            if process_id in self.active_processes:
                mp_process = self.active_processes[process_id]
                
                try:
                    mp_process.terminate()
                    mp_process.join(timeout=timeout)
                    
                    if mp_process.is_alive():
                        mp_process.kill()
                        mp_process.join()
                    
                    del self.active_processes[process_id]
                    process._status = 'stopped'
                    process._end_time = time.time()
                    
                    self.persistence.update_process(process_id, {
                        'status': 'stopped',
                        'end_time': time.time()
                    })
                    
                    self.console.success(f"تم إيقاف العملية: {process_id}")
                    return True
                    
                except Exception as e:
                    self.console.error(f"خطأ في إيقاف العملية {process_id}: {e}")
                    return False
            else:
                self.console.warning(f"العملية {process_id} غير نشطة")
                return True

    def restart_process(self, process_id: str) -> bool:
        self.console.info(f"إعادة تشغيل العملية: {process_id}")
        self.stop_process(process_id)
        time.sleep(2)
        return self.start_process(process_id)

    def get_process_status(self, process_id: str) -> Optional[Dict[str, Any]]:
        if process_id not in self.processes:
            return None
        
        process = self.processes[process_id]
        status = {
            'id': process_id,
            'status': process._status,
            'pid': process._pid,
            'start_time': process._start_time,
            'end_time': process._end_time,
            'running_time': None
        }
        
        if process._start_time and process._status == 'running':
            status['running_time'] = time.time() - process._start_time
        
        if process._pid and psutil.pid_exists(process._pid):
            try:
                ps_process = psutil.Process(process._pid)
                status['cpu_percent'] = ps_process.cpu_percent()
                status['memory_info'] = ps_process.memory_info()._asdict()
                status['num_threads'] = ps_process.num_threads()
            except psutil.NoSuchProcess:
                status['status'] = 'dead'
        
        return status

    def list_processes(self) -> List[Dict[str, Any]]:
        processes_info = []
        for process_id in self.processes:
            process_info = self.get_process_status(process_id)
            if process_info:
                processes_info.append(process_info)
        return processes_info

    def stop_all_processes(self):
        self.console.info("إيقاف جميع العمليات...")
        process_ids = list(self.processes.keys())
        for process_id in process_ids:
            self.stop_process(process_id)
        
        if self._monitor_thread and self._monitor_thread.is_alive():
            self._monitoring = False
            self._monitor_thread.join(timeout=5)

    def _start_monitoring(self):
        if self._monitoring:
            return
        
        self._monitoring = True
        self._monitor_thread = threading.Thread(target=self._monitor_processes, daemon=True)
        self._monitor_thread.start()
        self.console.info("بدء مراقبة العمليات")

    def _monitor_processes(self):
        while self._monitoring:
            try:
                dead_processes = []
                
                for process_id, mp_process in list(self.active_processes.items()):
                    if not mp_process.is_alive():
                        dead_processes.append(process_id)
                
                for process_id in dead_processes:
                    self.console.warning(f"العملية {process_id} توقفت بشكل غير متوقع")
                    if process_id in self.active_processes:
                        del self.active_processes[process_id]
                    
                    if process_id in self.processes:
                        self.processes[process_id]._status = 'dead'
                        self.processes[process_id]._end_time = time.time()
                        
                        self.persistence.update_process(process_id, {
                            'status': 'dead',
                            'end_time': time.time()
                        })
                
                time.sleep(5)
                
            except Exception as e:
                self.console.error(f"خطأ في مراقبة العمليات: {e}")
                time.sleep(10)

    def cleanup_dead_processes(self):
        dead_process_ids = []
        for process_id, process in self.processes.items():
            if process._status in ['dead', 'failed', 'stopped']:
                dead_process_ids.append(process_id)
        
        for process_id in dead_process_ids:
            del self.processes[process_id]
            self.persistence.remove_process(process_id)
            self.console.info(f"تم تنظيف العملية: {process_id}")

    def get_system_stats(self) -> Dict[str, Any]:
        return {
            'cpu_percent': psutil.cpu_percent(interval=1),
            'memory': psutil.virtual_memory()._asdict(),
            'disk': psutil.disk_usage('/')._asdict(),
            'active_processes': len(self.active_processes),
            'total_processes': len(self.processes),
            'load_average': os.getloadavg() if hasattr(os, 'getloadavg') else None
        }

    def export_processes(self, filepath: str):
        data = {
            'processes': {},
            'export_time': time.time(),
            'version': '2.5.0'
        }
        
        for process_id, process in self.processes.items():
            data['processes'][process_id] = {
                'status': process._status,
                'pid': process._pid,
                'start_time': process._start_time,
                'end_time': process._end_time,
                'args': process.args,
                'kwargs': process.kwargs
            }
        
        with open(filepath, 'w') as f:
            json.dump(data, f, indent=2)
        
        self.console.success(f"تم تصدير العمليات إلى: {filepath}")



# حقوق الطبع والنشر محفوظة © 2025 mero - من أرض الزيتون فلسطين

from .fastapi_server import FastAPIServer
from .flask_server import FlaskServer  
from .django_server import DjangoServer
from .tornado_server import TornadoServer

__all__ = ['FastAPIServer', 'FlaskServer', 'DjangoServer', 'TornadoServer']



# حقوق الطبع والنشر محفوظة © 2025 mero - فلسطين الحبيبة
# هذا المشروع مطور بأيادي فلسطينية أصيلة من أجل خدمة المطورين العرب

from setuptools import setup, find_packages
import os
import time

def show_installation_progress():
    from rich.console import Console
    from rich.progress import Progress, SpinnerColumn, TextColumn, BarColumn, TaskProgressColumn
    from rich.panel import Panel
    from rich.text import Text
    
    console = Console()
    
    ascii_art = """
    ████████╗███████╗██████╗ ███╗   ███╗██╗   ██╗██╗  ██╗
    ╚══██╔══╝██╔════╝██╔══██╗████╗ ████║██║   ██║╚██╗██╔╝
       ██║   █████╗  ██████╔╝██╔████╔██║██║   ██║ ╚███╔╝ 
       ██║   ██╔══╝  ██╔══██╗██║╚██╔╝██║██║   ██║ ██╔██╗ 
       ██║   ███████╗██║  ██║██║ ╚═╝ ██║╚██████╔╝██╔╝ ██╗
       ╚═╝   ╚══════╝╚═╝  ╚═╝╚═╝     ╚═╝ ╚═════╝ ╚═╝  ╚═╝
    
    ██████╗ ██╗   ██╗███╗   ██╗███╗   ███╗███████╗██████╗  ██████╗ 
    ██╔══██╗██║   ██║████╗  ██║████╗ ████║██╔════╝██╔══██╗██╔═══██╗
    ██████╔╝██║   ██║██╔██╗ ██║██╔████╔██║█████╗  ██████╔╝██║   ██║
    ██╔══██╗██║   ██║██║╚██╗██║██║╚██╔╝██║██╔══╝  ██╔══██╗██║   ██║
    ██║  ██║╚██████╔╝██║ ╚████║██║ ╚═╝ ██║███████╗██║  ██║╚██████╔╝
    ╚═╝  ╚═╝ ╚═════╝ ╚═╝  ╚═══╝╚═╝     ╚═╝╚══════╝╚═╝  ╚═╝ ╚═════╝ 
    """
    
    console.print(Panel(ascii_art, style="bold green"))
    console.print("[bold magenta]مرحباً بك في مكتبة RunMero - بقوة فلسطين![/bold magenta]")
    
    with Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        BarColumn(),
        TaskProgressColumn(),
        console=console,
        transient=True,
    ) as progress:
        
        tasks = [
            ("تحميل المكونات الأساسية...", 30),
            ("تثبيت خوادم FastAPI و Uvicorn...", 25),
            ("تكوين Flask و Django...", 20),
            ("إعداد Tornado Server...", 15),
            ("تحسين Termux Environment...", 10),
            ("تفعيل خدمات الخلفية...", 15),
            ("التحقق من التوافق...", 10),
            ("إنهاء التثبيت...", 5)
        ]
        
        for description, steps in tasks:
            task = progress.add_task(description, total=steps)
            for i in range(steps):
                time.sleep(0.1)
                progress.update(task, advance=1)
    
    console.print("\n[bold green]✅ تم تثبيت RunMero بنجاح![/bold green]")
    console.print("[bold yellow]استخدم الأمر 'helpmero' لبدء الاستخدام[/bold yellow]")
    console.print("[dim]© 2025 mero - صنع بحب في فلسطين الحبيبة[/dim]")

def get_large_data():
    large_content = []
    base_templates = [
        "server_template_fastapi", "server_template_flask", "server_template_django",
        "server_template_tornado", "middleware_components", "security_modules",
        "database_connectors", "cache_systems", "monitoring_tools", "logging_frameworks"
    ]
    
    for template in base_templates:
        for i in range(1000):
            large_content.append(f"{template}_{i}" * 100)
    
    return "\n".join(large_content)

with open("runmero/data/large_data.py", "w") as f:
    f.write(f"LARGE_DATA = '''{get_large_data()}'''")

class CustomInstallCommand:
    def run(self):
        show_installation_progress()

setup(
    name="runmero",
    version="2.5.0",
    author="mero",
    author_email="mero@palestine.dev",
    description="مكتبة قوية لإدارة العمليات في الخلفية مع دعم متعدد الإطارات للخوادم في بيئة Termux",
    long_description="مكتبة RunMero - الحل الأمثل لتشغيل التطبيقات في الخلفية بدون انقطاع في بيئة Termux. تدعم FastAPI, Flask, Django, Tornado مع إدارة ذكية للعمليات والموارد. مطور بأيادي فلسطينية أصيلة.",
    long_description_content_type="text/plain",
    url="https://github.com/mero-palestine/runmero",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "fastapi>=0.104.0",
        "uvicorn[standard]>=0.24.0",
        "flask>=3.0.0",
        "django>=4.2.0",
        "tornado>=6.4.0",
        "psutil>=5.9.0",
        "rich>=13.7.0",
        "click>=8.1.0",
        "pydantic>=2.5.0",
        "aiofiles>=23.2.0",
        "watchdog>=3.0.0",
        "cryptography>=41.0.0",
        "requests>=2.31.0",
        "websockets>=12.0",
        "redis>=5.0.0",
        "celery>=5.3.0",
        "gunicorn>=21.2.0",
        "nginx>=1.0.0",
        "supervisor>=4.2.0"
    ],
    entry_points={
        'console_scripts': [
            'helpmero=runmero.cli.main:main',
            'runmero=runmero.cli.main:cli',
        ],
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Operating System :: POSIX :: Linux",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Internet :: WWW/HTTP :: HTTP Servers",
        "Topic :: System :: Systems Administration",
    ],
    python_requires=">=3.8",
    zip_safe=False,
    platforms=["linux", "android"],
)
